"""
数据处理工具模块

包括缓存、数据客户端、验证器和转换器
"""

# 从 cache 导入缓存相关功能
from .cache import (
    home_path,
    get_dir_size,
    empty_cache_path,
    DiskCache,
    disk_cache,
    clear_cache,
    clear_expired_cache,
)

# 从 client 导入数据客户端
from .client import (
    DataClient,
    set_url_token,
    get_url_token,
)

# 从 validators 导入验证函数
from .validators import (
    validate_dataframe_columns,
    validate_datetime_index,
    validate_numeric_column,
    validate_date_range,
    validate_no_duplicates,
    validate_weight_data,
)

# 从 converters 导入转换函数
from .converters import (
    to_standard_kline_format,
    pivot_weight_data,
    resample_to_period,
    normalize_symbol,
    convert_dict_to_dataframe,
    ensure_datetime_column,
    flatten_multiindex_columns,
)

__all__ = [
    # Cache
    'home_path',
    'get_dir_size',
    'empty_cache_path',
    'DiskCache',
    'disk_cache',
    'clear_cache',
    'clear_expired_cache',
    # Client
    'DataClient',
    'set_url_token',
    'get_url_token',
    # Validators
    'validate_dataframe_columns',
    'validate_datetime_index',
    'validate_numeric_column',
    'validate_date_range',
    'validate_no_duplicates',
    'validate_weight_data',
    # Converters
    'to_standard_kline_format',
    'pivot_weight_data',
    'resample_to_period',
    'normalize_symbol',
    'convert_dict_to_dataframe',
    'ensure_datetime_column',
    'flatten_multiindex_columns',
]



from _typeshed import Incomplete
from czsc.core import BarGenerator as BarGenerator, CZSC as CZSC, Position as Position, RawBar as RawBar, Signal as Signal
from czsc.traders.sig_parse import get_signals_freqs as get_signals_freqs
from czsc.utils.data.cache import home_path as home_path
from datetime import datetime
from typing import AnyStr, Callable

class CzscSignals:
    name: str
    cache: Incomplete
    kwargs: Incomplete
    signals_config: Incomplete
    bg: Incomplete
    symbol: Incomplete
    base_freq: Incomplete
    freqs: Incomplete
    kas: Incomplete
    s: Incomplete
    def __init__(self, bg: BarGenerator | None = None, **kwargs) -> None: ...
    def get_signals_by_conf(self): ...
    def take_snapshot(self, file_html=None, width: str = '1400px', height: str = '580px'): ...
    def open_in_browser(self, width: str = '1400px', height: str = '580px') -> None: ...
    def update_signals(self, bar: RawBar): ...

def generate_czsc_signals(bars: list[RawBar], signals_config: list[dict], sdt: AnyStr | datetime = '20170101', init_n: int = 500, df: bool = False, **kwargs): ...
def check_signals_acc(bars: list[RawBar], signals_config: list[dict], delta_days: int = 5, **kwargs) -> None: ...
def get_unique_signals(bars: list[RawBar], signals_config: list[dict], **kwargs): ...

class CzscTrader(CzscSignals):
    positions: Incomplete
    name: str
    def __init__(self, bg: BarGenerator | None = None, positions: list[Position] | None = None, ensemble_method: AnyStr | Callable = 'mean', **kwargs) -> None: ...
    def update(self, bar: RawBar) -> None: ...
    s: Incomplete
    def on_sig(self, sig: dict) -> None: ...
    def on_bar(self, bar: RawBar) -> None: ...
    @property
    def pos_changed(self) -> bool: ...
    def get_ensemble_pos(self, method: AnyStr | Callable = None) -> float: ...
    def get_position(self, name: str) -> Position | None: ...
    def take_snapshot(self, file_html=None, width: str = '1400px', height: str = '580px'): ...
    def get_ensemble_weight(self, method: AnyStr | Callable | None = None): ...
    def weight_backtest(self, **kwargs): ...



from collections import OrderedDict
from czsc import CZSC as CZSC
from czsc.core import BI as BI, Direction as Direction, FX as FX, Mark as Mark, ZS as ZS
from czsc.signals.tas import update_ma_cache as update_ma_cache, update_macd_cache as update_macd_cache
from czsc.traders.base import CzscSignals as CzscSignals
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements, get_zs_seq as get_zs_seq

def cxt_bi_base_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_fx_power_V221107(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_first_buy_V221126(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_first_sell_V221126(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_zhong_shu_gong_zhen_V221221(cat: CzscSignals, freq1: str = '日线', freq2: str = '60分钟', **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230222(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230224(c: CZSC, **kwargs): ...
def cxt_third_buy_V230228(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_double_zs_V230311(c: CZSC, **kwargs): ...
def cxt_second_bs_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_third_bs_V230318(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_third_bs_V230319(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230104(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230105(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230312(c: CZSC, **kwargs): ...
def cxt_bi_end_V230320(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230322(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230324(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_status_V230101(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_status_V230102(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_zdf_V230601(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230618(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_three_bi_V230618(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_five_bi_V230619(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_seven_bi_V230620(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_nine_bi_V230621(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_eleven_bi_V230622(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_range_oscillation_V230620(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_intraday_V230701(cat: CzscSignals, **kwargs) -> OrderedDict: ...
def cxt_ubi_end_V230816(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_end_V230815(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_stop_V230815(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_trend_V230824(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bi_trend_V230913(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_second_bs_V240524(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_overlap_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bs_V240526(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_bs_V240527(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_overlap_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240612(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240613(c: CZSC, **kwargs) -> OrderedDict: ...
def cxt_decision_V240614(c: CZSC, **kwargs) -> OrderedDict: ...



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/11/1 22:20
describe: 交易员（traders）：使用 CZSC 分析工具进行择时策略的开发，交易等
"""
from czsc.traders.base import CzscSignals, CzscTrader, generate_czsc_signals, check_signals_acc, get_unique_signals

from czsc.traders.performance import (
    PairsPerformance,
    combine_holds_and_pairs,
    combine_dates_and_pairs,
)
from czsc.traders.dummy import DummyBacktest
from czsc.traders.sig_parse import SignalsParser, get_signals_config, get_signals_freqs
from czsc.traders.weight_backtest import get_ensemble_weight, stoploss_by_direction

# 延迟加载的属性映射
_LAZY_ATTRS = {
    'OpensOptimize': ('czsc.traders.optimize', 'OpensOptimize'),
    'ExitsOptimize': ('czsc.traders.optimize', 'ExitsOptimize'),
    'RedisWeightsClient': ('czsc.traders.rwc', 'RedisWeightsClient'),
    'get_strategy_mates': ('czsc.traders.rwc', 'get_strategy_mates'),
    'get_heartbeat_time': ('czsc.traders.rwc', 'get_heartbeat_time'),
    'clear_strategy': ('czsc.traders.rwc', 'clear_strategy'),
    'get_strategy_weights': ('czsc.traders.rwc', 'get_strategy_weights'),
    'get_strategy_latest': ('czsc.traders.rwc', 'get_strategy_latest'),
}


def __getattr__(name):
    if name in _LAZY_ATTRS:
        import importlib
        mod_path, attr_name = _LAZY_ATTRS[name]
        module = importlib.import_module(mod_path)
        attr = getattr(module, attr_name)
        globals()[name] = attr
        return attr
    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")



from collections import OrderedDict
from czsc.core import CZSC as CZSC
from czsc.signals.tas import update_cci_cache as update_cci_cache, update_kdj_cache as update_kdj_cache, update_ma_cache as update_ma_cache, update_sar_cache as update_sar_cache
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def coo_td_V221110(c: CZSC, **kwargs) -> OrderedDict: ...
def coo_td_V221111(c: CZSC, **kwargs) -> OrderedDict: ...
def coo_cci_V230323(c: CZSC, **kwargs) -> OrderedDict: ...
def coo_kdj_V230322(c: CZSC, **kwargs) -> OrderedDict: ...
def coo_sar_V230325(c: CZSC, **kwargs) -> OrderedDict: ...



import pandas as pd
from .base import apply_stats_style as apply_stats_style, generate_component_key as generate_component_key, safe_import_weight_backtest as safe_import_weight_backtest
from loguru import logger as logger

def show_optuna_study(study, key=None, **kwargs): ...
def show_czsc_trader(trader, max_k_num: int = 300, key=None, **kwargs): ...
def show_strategies_recent(df, **kwargs): ...
def show_returns_contribution(df, returns=None, max_returns: int = 100, key=None) -> None: ...
def show_symbols_bench(df: pd.DataFrame, **kwargs): ...
def show_quarterly_effect(returns: pd.Series, key=None): ...
def show_multi_backtest(wbs: dict, **kwargs): ...
def show_cta_periods_classify(df: pd.DataFrame, **kwargs): ...
def show_volatility_classify(df: pd.DataFrame, kind: str = 'ts', **kwargs): ...
def show_portfolio(df: pd.DataFrame, portfolio: str, benchmark: str | None = None, **kwargs): ...
def show_turnover_rate(df: pd.DataFrame): ...
def show_stats_compare(df: pd.DataFrame, **kwargs): ...
def show_symbol_penalty(df: pd.DataFrame, n: int = 3, **kwargs): ...



import pandas as pd
from .base import safe_import_daily_performance as safe_import_daily_performance, safe_import_weight_backtest as safe_import_weight_backtest

def show_weight_distribution(dfw, abs_weight: bool = True, **kwargs): ...
def show_weight_backtest(dfw, **kwargs): ...
def show_holds_backtest(df, **kwargs) -> None: ...
def show_stoploss_by_direction(dfw, **kwargs) -> None: ...
def show_backtest_by_thresholds(df: pd.DataFrame, out_sample_sdt, **kwargs): ...
def show_backtest_by_year(df: pd.DataFrame, **kwargs): ...
def show_backtest_by_symbol(df: pd.DataFrame, **kwargs): ...
def show_long_short_backtest(df: pd.DataFrame, **kwargs): ...
def show_comprehensive_weight_backtest(df: pd.DataFrame, **kwargs): ...



import pandas as pd
from czsc.core import CZSC as CZSC, RawBar as RawBar

def calculate_bi_info(bars: list[RawBar], **kwargs) -> pd.DataFrame: ...
def symbols_bi_infos(symbols, read_bars, freq: str = '5分钟', sdt: str = '20130101', edt: str = '20190101', **kwargs) -> pd.DataFrame: ...



"""
数据验证工具

提取的通用数据验证逻辑
"""

import pandas as pd
from typing import List, Optional


def validate_dataframe_columns(
    df: pd.DataFrame,
    required_columns: List[str],
    name: str = "DataFrame"
) -> None:
    """验证DataFrame包含所有必需的列

    :param df: 要验证的DataFrame
    :param required_columns: 必需的列名列表
    :param name: DataFrame的名称，用于错误消息
    :raises ValueError: 如果缺少必需的列
    """
    missing_columns = set(required_columns) - set(df.columns)
    if missing_columns:
        raise ValueError(
            f"{name} 缺少必需的列: {missing_columns}. "
            f"当前列: {list(df.columns)}"
        )


def validate_datetime_index(
    df: pd.DataFrame,
    name: str = "DataFrame"
) -> None:
    """验证DataFrame的索引是DatetimeIndex

    :param df: 要验证的DataFrame
    :param name: DataFrame的名称，用于错误消息
    :raises ValueError: 如果索引不是DatetimeIndex
    """
    if not isinstance(df.index, pd.DatetimeIndex):
        raise ValueError(
            f"{name} 的索引必须是 DatetimeIndex, "
            f"当前类型: {type(df.index)}"
        )


def validate_numeric_column(
    df: pd.DataFrame,
    column: str,
    name: str = "DataFrame"
) -> None:
    """验证指定列是数值类型

    :param df: 要验证的DataFrame
    :param column: 列名
    :param name: DataFrame的名称，用于错误消息
    :raises ValueError: 如果列不是数值类型
    """
    if column not in df.columns:
        raise ValueError(f"{name} 中不存在列: {column}")
    
    if not pd.api.types.is_numeric_dtype(df[column]):
        raise ValueError(
            f"{name} 的列 '{column}' 必须是数值类型, "
            f"当前类型: {df[column].dtype}"
        )


def validate_date_range(
    df: pd.DataFrame,
    start_date: Optional[pd.Timestamp] = None,
    end_date: Optional[pd.Timestamp] = None,
    name: str = "DataFrame"
) -> None:
    """验证DataFrame的日期范围

    :param df: 要验证的DataFrame (必须有DatetimeIndex)
    :param start_date: 最小允许日期
    :param end_date: 最大允许日期
    :param name: DataFrame的名称，用于错误消息
    :raises ValueError: 如果日期范围无效
    """
    validate_datetime_index(df, name)
    
    if start_date is not None and df.index.min() < start_date:
        raise ValueError(
            f"{name} 包含早于 {start_date} 的数据, "
            f"最早日期: {df.index.min()}"
        )
    
    if end_date is not None and df.index.max() > end_date:
        raise ValueError(
            f"{name} 包含晚于 {end_date} 的数据, "
            f"最晚日期: {df.index.max()}"
        )


def validate_no_duplicates(
    df: pd.DataFrame,
    subset: Optional[List[str]] = None,
    name: str = "DataFrame"
) -> None:
    """验证DataFrame中没有重复行

    :param df: 要验证的DataFrame
    :param subset: 用于检查重复的列的子集
    :param name: DataFrame的名称，用于错误消息
    :raises ValueError: 如果存在重复行
    """
    duplicates = df.duplicated(subset=subset)
    if duplicates.any():
        num_duplicates = duplicates.sum()
        from loguru import logger
        logger.warning(f"{name} 包含 {num_duplicates} 行重复数据")
        raise ValueError(
            f"{name} 包含 {num_duplicates} 行重复数据. "
            f"请检查数据质量"
        )


def validate_weight_data(df: pd.DataFrame) -> None:
    """验证权重数据格式

    权重数据应包含 dt, symbol, weight 三列

    :param df: 权重数据DataFrame
    :raises ValueError: 如果数据格式无效
    """
    validate_dataframe_columns(
        df,
        required_columns=['dt', 'symbol', 'weight'],
        name="权重数据"
    )
    
    # 验证 dt 列可以转换为日期
    try:
        pd.to_datetime(df['dt'])
    except Exception as e:
        raise ValueError(f"'dt' 列无法转换为日期格式: {e}")
    
    # 验证 weight 列是数值类型
    validate_numeric_column(df, 'weight', "权重数据")



"""
PDF 报告构建器

提供灵活的 PDF 报告生成功能，支持链式调用和按需添加内容元素。
使用 reportlab 库生成专业的量化金融 PDF 报告，支持中文文本、Plotly 图表嵌入和数据表格。

v2: 增强 PDF 专属特性 —— 目录、页面填充优化、图表缩放控制。
"""

import io
import os
import tempfile
from datetime import datetime
from typing import List, Dict, Any, Optional, Union

import pandas as pd
from reportlab.lib import colors
from reportlab.lib.colors import HexColor
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
from reportlab.lib.pagesizes import A4, landscape
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import cm, mm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.cidfonts import UnicodeCIDFont
from reportlab.platypus import (
    SimpleDocTemplate, Paragraph, Spacer, Table, TableStyle,
    Image, PageBreak, HRFlowable, KeepTogether,
)

# ---------------------------------------------------------------------------
# 注册中文字体（reportlab 内置 CID 字体，无需外部字体文件）
# ---------------------------------------------------------------------------
pdfmetrics.registerFont(UnicodeCIDFont("STSong-Light"))

# ---------------------------------------------------------------------------
# 颜色常量（与 HTML 报告主题一致）
# ---------------------------------------------------------------------------
COLOR_PRIMARY = HexColor("#007bff")
COLOR_SUCCESS = HexColor("#28a745")
COLOR_DANGER = HexColor("#dc3545")
COLOR_WARNING = HexColor("#ffc107")
COLOR_SECONDARY = HexColor("#6c757d")
COLOR_LIGHT = HexColor("#f8f9fa")
COLOR_DARK = HexColor("#212529")
COLOR_BORDER = HexColor("#dee2e6")

COLOR_SUCCESS_BG = HexColor("#d4edda")
COLOR_DANGER_BG = HexColor("#f8d7da")
COLOR_PRIMARY_BG = HexColor("#cce5ff")

# ---------------------------------------------------------------------------
# 页面布局常量
# ---------------------------------------------------------------------------
PAGE_SIZE = landscape(A4)
PAGE_WIDTH, PAGE_HEIGHT = PAGE_SIZE
MARGIN_TOP = 1.5 * cm
MARGIN_BOTTOM = 1.5 * cm
MARGIN_LEFT = 2 * cm
MARGIN_RIGHT = 2 * cm
CONTENT_WIDTH = PAGE_WIDTH - MARGIN_LEFT - MARGIN_RIGHT
CONTENT_HEIGHT = PAGE_HEIGHT - MARGIN_TOP - MARGIN_BOTTOM
CM_TO_PX = 96 / 2.54  # 96 DPI / 2.54 cm per inch ≈ 37.795
CARD_SPACING = 4  # 指标卡片间距（pt）

FONT_NAME = "STSong-Light"
FONT_NAME_BOLD = "STSong-Light"  # CID 字体无独立粗体，用同一字体


def _build_styles() -> Dict[str, ParagraphStyle]:
    """构建报告中使用的段落样式集合。"""
    base = getSampleStyleSheet()
    styles: Dict[str, ParagraphStyle] = {}

    styles["title"] = ParagraphStyle(
        "PDFTitle", parent=base["Title"], fontName=FONT_NAME, fontSize=24,
        leading=32, textColor=COLOR_DARK, alignment=TA_CENTER, spaceAfter=4,
    )
    styles["subtitle"] = ParagraphStyle(
        "PDFSubtitle", parent=base["Normal"], fontName=FONT_NAME, fontSize=11,
        leading=16, textColor=COLOR_SECONDARY, alignment=TA_CENTER, spaceAfter=8,
    )
    styles["section_title"] = ParagraphStyle(
        "PDFSectionTitle", parent=base["Heading2"], fontName=FONT_NAME, fontSize=14,
        leading=20, textColor=COLOR_PRIMARY, spaceBefore=10, spaceAfter=6,
        borderWidth=0, borderPadding=0, borderColor=COLOR_PRIMARY,
    )
    styles["body"] = ParagraphStyle(
        "PDFBody", parent=base["Normal"], fontName=FONT_NAME, fontSize=9,
        leading=14, textColor=COLOR_DARK, spaceAfter=6,
    )
    styles["badge"] = ParagraphStyle(
        "PDFBadge", parent=base["Normal"], fontName=FONT_NAME, fontSize=8,
        leading=12, textColor=COLOR_SECONDARY, alignment=TA_CENTER,
    )
    styles["metric_value"] = ParagraphStyle(
        "PDFMetricValue", parent=base["Normal"], fontName=FONT_NAME, fontSize=16,
        leading=22, alignment=TA_CENTER, spaceAfter=2,
    )
    styles["metric_label"] = ParagraphStyle(
        "PDFMetricLabel", parent=base["Normal"], fontName=FONT_NAME, fontSize=8,
        leading=12, textColor=COLOR_SECONDARY, alignment=TA_CENTER,
    )
    styles["footer"] = ParagraphStyle(
        "PDFFooter", parent=base["Normal"], fontName=FONT_NAME, fontSize=8,
        leading=12, textColor=COLOR_SECONDARY, alignment=TA_CENTER,
    )
    styles["table_header"] = ParagraphStyle(
        "PDFTableHeader", parent=base["Normal"], fontName=FONT_NAME, fontSize=8,
        leading=12, textColor=colors.white, alignment=TA_CENTER,
    )
    styles["table_cell"] = ParagraphStyle(
        "PDFTableCell", parent=base["Normal"], fontName=FONT_NAME, fontSize=8,
        leading=12, textColor=COLOR_DARK, alignment=TA_CENTER,
    )
    styles["toc_title"] = ParagraphStyle(
        "PDFTocTitle", parent=base["Heading1"], fontName=FONT_NAME, fontSize=18,
        leading=24, textColor=COLOR_DARK, alignment=TA_LEFT, spaceBefore=6, spaceAfter=12,
    )
    styles["toc_entry"] = ParagraphStyle(
        "PDFTocEntry", parent=base["Normal"], fontName=FONT_NAME, fontSize=11,
        leading=20, textColor=COLOR_DARK, alignment=TA_LEFT,
        leftIndent=12, spaceBefore=2, spaceAfter=2,
    )
    styles["toc_entry_sub"] = ParagraphStyle(
        "PDFTocEntrySub", parent=base["Normal"], fontName=FONT_NAME, fontSize=10,
        leading=18, textColor=COLOR_SECONDARY, alignment=TA_LEFT,
        leftIndent=30, spaceBefore=1, spaceAfter=1,
    )
    return styles


# ---------------------------------------------------------------------------
# 辅助函数
# ---------------------------------------------------------------------------

def _fig_to_image_bytes(fig, width: float, height: float) -> bytes:
    """将 Plotly Figure 转换为 PNG 字节流。

    :param fig: plotly.graph_objects.Figure 对象
    :param width: 图片宽度（像素）
    :param height: 图片高度（像素）
    :return: PNG 格式字节流
    """
    return fig.to_image(format="png", width=int(width), height=int(height), scale=2)


def _make_metric_cell(value_text: str, label_text: str, is_positive: Optional[bool],
                      styles: Dict[str, ParagraphStyle], cell_width: float) -> Table:
    """创建单个指标卡片（作为内嵌 Table 返回）。

    :param value_text: 指标值文本
    :param label_text: 指标标签文本
    :param is_positive: 正向/负向/中性 (True/False/None)
    :param styles: 样式字典
    :param cell_width: 单元格宽度
    :return: reportlab Table 对象
    """
    if is_positive is True:
        val_color = COLOR_SUCCESS
        bg_color = COLOR_SUCCESS_BG
    elif is_positive is False:
        val_color = COLOR_DANGER
        bg_color = COLOR_DANGER_BG
    else:
        val_color = COLOR_PRIMARY
        bg_color = COLOR_PRIMARY_BG

    val_style = ParagraphStyle(
        "mv", parent=styles["metric_value"], textColor=val_color,
    )
    val_para = Paragraph(str(value_text), val_style)
    lbl_para = Paragraph(str(label_text), styles["metric_label"])

    card = Table([[val_para], [lbl_para]], colWidths=[cell_width], rowHeights=[28, 18])
    card.setStyle(TableStyle([
        ("BACKGROUND", (0, 0), (-1, -1), bg_color),
        ("BOX", (0, 0), (-1, -1), 0.5, COLOR_BORDER),
        ("TOPPADDING", (0, 0), (-1, -1), 4),
        ("BOTTOMPADDING", (0, 0), (-1, -1), 4),
        ("LEFTPADDING", (0, 0), (-1, -1), 4),
        ("RIGHTPADDING", (0, 0), (-1, -1), 4),
        ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
        ("ALIGN", (0, 0), (-1, -1), "CENTER"),
        ("ROUNDEDCORNERS", [4, 4, 4, 4]),
    ]))
    return card


# ---------------------------------------------------------------------------
# PdfReportBuilder 类
# ---------------------------------------------------------------------------

class PdfReportBuilder:
    """PDF 报告构建器

    支持链式调用，按需添加各种元素，生成专业的量化金融 PDF 报告。

    v2 增强:
    - add_toc(): 自动生成目录页（基于已添加的 section_title 收集）
    - add_page_break(): 显式分页
    - add_chart(): 支持 fit_page 模式自动适配页面
    - 每个 section_title 自动收集用于目录生成

    示例用法::

        builder = PdfReportBuilder(title="策略回测报告", author="CZSC")
        builder.add_header({"日期": "2024-01-01", "版本": "v1.0"}, subtitle="沪深300增强策略") \\
               .add_metrics([{"label": "年化收益", "value": "15.3%", "is_positive": True}]) \\
               .add_chart(fig, title="累计收益曲线") \\
               .add_table(df, title="持仓明细") \\
               .add_section("风险提示", "本报告仅供参考。") \\
               .add_footer() \\
               .save("report.pdf")
    """

    def __init__(self, title: str = "PDF 报告", author: str = "CZSC"):
        """初始化 PDF 报告构建器

        :param title: 报告标题
        :param author: 报告作者
        """
        self.title = title
        self.author = author
        self.created_at = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self._styles = _build_styles()
        self._elements: List[Any] = []  # Platypus flowable 列表
        self._footer_text: Optional[str] = None
        self._temp_files: List[str] = []  # 临时文件路径，save/render 后清理
        self._toc_entries: List[Dict[str, Any]] = []  # 目录条目: {"title": str, "level": int}

    # ----- 链式方法 --------------------------------------------------------

    def add_header(self, params: Dict[str, str], subtitle: str = None) -> "PdfReportBuilder":
        """添加报告头部区域

        :param params: 参数字典，如 {"日期": "2024-01-01", "版本": "v1.0"}
        :param subtitle: 副标题
        :return: self，支持链式调用
        """
        # 标题
        self._elements.append(Paragraph(self.title, self._styles["title"]))

        # 副标题
        if subtitle:
            self._elements.append(Paragraph(subtitle, self._styles["subtitle"]))

        # 参数徽章行 —— 均匀分布到整个内容宽度
        if params:
            badge_cells = []
            for key, value in params.items():
                badge_cells.append(Paragraph(f"{key}: {value}", self._styles["badge"]))

            n_badges = len(badge_cells)
            badge_width = CONTENT_WIDTH / max(n_badges, 1)
            badge_table = Table([badge_cells], colWidths=[badge_width] * n_badges)
            badge_table.setStyle(TableStyle([
                ("BACKGROUND", (0, 0), (-1, -1), COLOR_LIGHT),
                ("BOX", (0, 0), (-1, -1), 0.5, COLOR_BORDER),
                ("INNERGRID", (0, 0), (-1, -1), 0.25, COLOR_BORDER),
                ("TOPPADDING", (0, 0), (-1, -1), 4),
                ("BOTTOMPADDING", (0, 0), (-1, -1), 4),
                ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
                ("ALIGN", (0, 0), (-1, -1), "CENTER"),
                ("ROUNDEDCORNERS", [3, 3, 3, 3]),
            ]))
            self._elements.append(Spacer(1, 6))
            self._elements.append(badge_table)

        # 分隔线
        self._elements.append(Spacer(1, 8))
        self._elements.append(HRFlowable(width="100%", thickness=1, color=COLOR_BORDER, spaceAfter=10))
        return self

    def add_toc(self, title: str = "目 录") -> "PdfReportBuilder":
        """添加目录页

        基于 ``_toc_entries`` 列表生成一个简洁的目录页，然后自动分页。
        目录条目在调用 ``add_chart`` / ``add_table`` / ``add_section`` 等方法时自动收集。

        **注意**: 应在所有内容添加完毕后、调用 ``render()`` / ``save()`` 之前调用此方法；
        也可以先添加内容，最后通过 ``_insert_toc_at(pos)`` 插入到指定位置。

        :param title: 目录区域标题
        :return: self，支持链式调用
        """
        toc_elements: List[Any] = []
        toc_elements.append(Paragraph(title, self._styles["toc_title"]))
        toc_elements.append(Spacer(1, 8))

        idx = 1
        for entry in self._toc_entries:
            level = entry.get("level", 1)
            entry_title = entry["title"]
            bookmark_key = entry.get("key", "")
            
            if level == 1:
                text = f'<a href="#{bookmark_key}" color="black">{idx}. {entry_title}</a>'
                toc_elements.append(Paragraph(text, self._styles["toc_entry"]))
                idx += 1
            else:
                text = f'<a href="#{bookmark_key}" color="black">   ● {entry_title}</a>'
                toc_elements.append(Paragraph(text, self._styles["toc_entry_sub"]))

        toc_elements.append(PageBreak())
        self._elements.extend(toc_elements)
        return self

    def insert_toc_after_header(self, title: str = "目 录", add_page_break: bool = True) -> "PdfReportBuilder":
        """在头部（header + metrics）之后插入目录页

        会自动检测第一个 HRFlowable（头部结束标记）的位置，并在其后插入目录。

        :param title: 目录区域标题
        :param add_page_break: 是否在目录后添加分页符，默认 True
        :return: self，支持链式调用
        """
        # 查找头部结束位置（第一个 HRFlowable 之后）
        insert_pos = 0
        for i, elem in enumerate(self._elements):
            if isinstance(elem, HRFlowable):
                insert_pos = i + 1
                break
        if insert_pos == 0:
            insert_pos = len(self._elements)

        toc_elements: List[Any] = []
        toc_elements.append(Spacer(1, 6))
        toc_elements.append(Paragraph(title, self._styles["toc_title"]))
        toc_elements.append(Spacer(1, 8))
        toc_elements.append(HRFlowable(width="100%", thickness=0.5, color=COLOR_BORDER, spaceAfter=6))

        idx = 1
        for entry in self._toc_entries:
            level = entry.get("level", 1)
            entry_title = entry["title"]
            bookmark_key = entry.get("key", "")
            
            if level == 1:
                # 带序号和虚线分隔
                text = f'<a href="#{bookmark_key}" color="black"><b>{idx}.</b>  {entry_title}</a>'
                toc_elements.append(Paragraph(text, self._styles["toc_entry"]))
                idx += 1
            else:
                text = f'<a href="#{bookmark_key}" color="black">● {entry_title}</a>'
                toc_elements.append(Paragraph(text, self._styles["toc_entry_sub"]))

        toc_elements.append(Spacer(1, 10))
        toc_elements.append(HRFlowable(width="100%", thickness=0.5, color=COLOR_BORDER, spaceAfter=6))
        if add_page_break:
            toc_elements.append(PageBreak())

        # 插入到指定位置
        for j, elem in enumerate(toc_elements):
            self._elements.insert(insert_pos + j, elem)

        return self

    def add_page_break(self) -> "PdfReportBuilder":
        """添加分页符

        :return: self，支持链式调用
        """
        self._elements.append(PageBreak())
        return self

    def add_metrics(self, metrics: List[Dict[str, Any]], title: str = "核心绩效指标") -> "PdfReportBuilder":
        """添加绩效指标卡片

        :param metrics: 指标列表，每个元素为 {"label": str, "value": str, "is_positive": bool/None}
        :param title: 区域标题
        :return: self，支持链式调用
        """
        self._elements.append(Paragraph(title, self._styles["section_title"]))

        cols_per_row = 5
        card_width = (CONTENT_WIDTH - (cols_per_row - 1) * CARD_SPACING) / cols_per_row

        rows: List[list] = []
        current_row: list = []
        for m in metrics:
            card = _make_metric_cell(
                value_text=str(m.get("value", "")),
                label_text=str(m.get("label", "")),
                is_positive=m.get("is_positive"),
                styles=self._styles,
                cell_width=card_width,
            )
            current_row.append(card)
            if len(current_row) == cols_per_row:
                rows.append(current_row)
                current_row = []
        if current_row:
            # 补齐空单元格
            while len(current_row) < cols_per_row:
                current_row.append("")
            rows.append(current_row)

        grid = Table(rows, colWidths=[card_width + CARD_SPACING] * cols_per_row)
        grid.setStyle(TableStyle([
            ("VALIGN", (0, 0), (-1, -1), "TOP"),
            ("LEFTPADDING", (0, 0), (-1, -1), 2),
            ("RIGHTPADDING", (0, 0), (-1, -1), 2),
            ("TOPPADDING", (0, 0), (-1, -1), 3),
            ("BOTTOMPADDING", (0, 0), (-1, -1), 3),
        ]))
        self._elements.append(grid)
        self._elements.append(Spacer(1, 8))
        return self

    def _add_bookmark_and_title(self, title: str, level: int = 1) -> None:
        """添加书签锚点和标题段落

        :param title: 标题文本
        :param level: 目录层级
        """
        # 生成唯一的书签 key
        bookmark_key = f"bm_{len(self._toc_entries)}"
        self._toc_entries.append({"title": title, "level": level, "key": bookmark_key})

        # 使用 a 标签创建内部锚点
        anchor_text = f'<a name="{bookmark_key}"/>{title}'
        self._elements.append(Paragraph(anchor_text, self._styles["section_title"]))

    def add_chart(self, fig_or_image, title: str = "图表",
                  height: float = None, fit_page: bool = False,
                  aspect_ratio: float = 0.55) -> "PdfReportBuilder":
        """添加图表

        支持 Plotly Figure 对象（通过 kaleido 转为 PNG）或图片文件路径。

        :param fig_or_image: Plotly Figure 对象 或 图片文件路径字符串
        :param title: 图表标题
        :param height: 图表高度（单位：cm）。若为 None 则根据 aspect_ratio 自动计算
        :param fit_page: 若为 True，图表将尽量填充当前页面的剩余空间（忽略 height 参数）
        :param aspect_ratio: 高度/宽度比例，默认 0.55，仅在 height=None 且 fit_page=False 时生效
        :return: self，支持链式调用
        """
        self._add_bookmark_and_title(title, level=1)

        img_width = CONTENT_WIDTH

        if fit_page:
            # 适配页面模式：使用可用内容高度减去标题和间距的估计值
            # 标题行约 30pt, spacer 约 8pt，保留 20pt 边距
            available_height = CONTENT_HEIGHT - 58
            img_height = min(available_height, img_width * aspect_ratio)
        elif height is not None:
            img_height = height * cm
        else:
            img_height = img_width * aspect_ratio

        if isinstance(fig_or_image, str):
            # 图片文件路径
            img = Image(fig_or_image, width=img_width, height=img_height)
        else:
            # Plotly Figure → PNG bytes
            px_width = int(img_width / cm * CM_TO_PX)
            px_height = int(img_height / cm * CM_TO_PX)
            png_bytes = _fig_to_image_bytes(fig_or_image, width=px_width, height=px_height)

            tmp = tempfile.NamedTemporaryFile(suffix=".png", delete=False)
            tmp.write(png_bytes)
            tmp.close()
            self._temp_files.append(tmp.name)
            img = Image(tmp.name, width=img_width, height=img_height)

        # 包裹图表在带边框的 Table 中
        chart_table = Table([[img]], colWidths=[CONTENT_WIDTH])
        chart_table.setStyle(TableStyle([
            ("BOX", (0, 0), (-1, -1), 0.5, COLOR_BORDER),
            ("TOPPADDING", (0, 0), (-1, -1), 2),
            ("BOTTOMPADDING", (0, 0), (-1, -1), 2),
            ("LEFTPADDING", (0, 0), (-1, -1), 2),
            ("RIGHTPADDING", (0, 0), (-1, -1), 2),
            ("ALIGN", (0, 0), (-1, -1), "CENTER"),
            ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ("ROUNDEDCORNERS", [4, 4, 4, 4]),
        ]))
        self._elements.append(chart_table)
        self._elements.append(Spacer(1, 4))
        return self

    def add_table(self, df: pd.DataFrame, title: str = "数据表",
                  max_rows: int = None) -> "PdfReportBuilder":
        """添加数据表格

        :param df: pandas DataFrame
        :param title: 表格标题
        :param max_rows: 最大显示行数，None 表示全部显示
        :return: self，支持链式调用
        """
        if df.empty:
            return self

        if max_rows and len(df) > max_rows:
            df = df.head(max_rows)

        self._add_bookmark_and_title(title, level=2)

        # 构建表格数据：表头 + 数据行，每个单元格用 Paragraph 包裹以支持中文和自动换行
        header_row = [Paragraph(str(c), self._styles["table_header"]) for c in df.columns]
        data_rows = []
        for _, row in df.iterrows():
            data_rows.append([Paragraph(str(v), self._styles["table_cell"]) for v in row])

        table_data = [header_row] + data_rows
        n_cols = len(df.columns)
        col_width = CONTENT_WIDTH / n_cols

        tbl = Table(table_data, colWidths=[col_width] * n_cols, repeatRows=1)

        # 表格样式：表头深蓝背景 + 交替行颜色
        style_commands = [
            # 表头
            ("BACKGROUND", (0, 0), (-1, 0), COLOR_PRIMARY),
            ("TEXTCOLOR", (0, 0), (-1, 0), colors.white),
            ("FONTNAME", (0, 0), (-1, -1), FONT_NAME),
            ("FONTSIZE", (0, 0), (-1, 0), 8),
            ("FONTSIZE", (0, 1), (-1, -1), 8),
            ("ALIGN", (0, 0), (-1, -1), "CENTER"),
            ("VALIGN", (0, 0), (-1, -1), "MIDDLE"),
            ("TOPPADDING", (0, 0), (-1, -1), 4),
            ("BOTTOMPADDING", (0, 0), (-1, -1), 4),
            ("LEFTPADDING", (0, 0), (-1, -1), 3),
            ("RIGHTPADDING", (0, 0), (-1, -1), 3),
            # 边框
            ("BOX", (0, 0), (-1, -1), 0.5, COLOR_BORDER),
            ("LINEBELOW", (0, 0), (-1, 0), 1, COLOR_PRIMARY),
            ("INNERGRID", (0, 0), (-1, -1), 0.25, COLOR_BORDER),
        ]
        # 交替行背景色
        for i in range(1, len(table_data)):
            bg = COLOR_LIGHT if i % 2 == 0 else colors.white
            style_commands.append(("BACKGROUND", (0, i), (-1, i), bg))

        tbl.setStyle(TableStyle(style_commands))
        self._elements.append(tbl)
        self._elements.append(Spacer(1, 8))
        return self

    def add_section(self, title: str, content: str) -> "PdfReportBuilder":
        """添加文本章节

        :param title: 章节标题
        :param content: 章节内容（纯文本或简单 HTML）
        :return: self，支持链式调用
        """
        self._add_bookmark_and_title(title, level=2)
        self._elements.append(Paragraph(content, self._styles["body"]))
        self._elements.append(Spacer(1, 6))
        return self

    def add_footer(self, text: str = None) -> "PdfReportBuilder":
        """设置页脚文本

        :param text: 页脚文本，None 则使用默认文本
        :return: self，支持链式调用
        """
        if text is None:
            text = f"由 CZSC 缠中说禅技术分析工具生成 | 作者: {self.author} | 生成时间: {self.created_at}"
        self._footer_text = text
        return self

    # ----- 页面模板回调 ----------------------------------------------------

    def _draw_page_header(self, canvas, doc):
        """在每页顶部绘制页眉。"""
        canvas.saveState()
        canvas.setFont(FONT_NAME, 8)
        canvas.setFillColor(COLOR_SECONDARY)
        canvas.drawString(MARGIN_LEFT, PAGE_HEIGHT - MARGIN_TOP + 6, self.title)
        canvas.drawRightString(PAGE_WIDTH - MARGIN_RIGHT, PAGE_HEIGHT - MARGIN_TOP + 6, self.created_at)
        canvas.setStrokeColor(COLOR_BORDER)
        canvas.setLineWidth(0.5)
        canvas.line(MARGIN_LEFT, PAGE_HEIGHT - MARGIN_TOP + 2, PAGE_WIDTH - MARGIN_RIGHT, PAGE_HEIGHT - MARGIN_TOP + 2)
        canvas.restoreState()

    def _draw_page_footer(self, canvas, doc):
        """在每页底部绘制页脚和页码。"""
        canvas.saveState()
        canvas.setStrokeColor(COLOR_BORDER)
        canvas.setLineWidth(0.5)
        canvas.line(MARGIN_LEFT, MARGIN_BOTTOM - 2, PAGE_WIDTH - MARGIN_RIGHT, MARGIN_BOTTOM - 2)

        canvas.setFont(FONT_NAME, 8)
        canvas.setFillColor(COLOR_SECONDARY)
        footer = self._footer_text or ""
        canvas.drawString(MARGIN_LEFT, MARGIN_BOTTOM - 14, footer)
        canvas.drawRightString(PAGE_WIDTH - MARGIN_RIGHT, MARGIN_BOTTOM - 14, f"第 {doc.page} 页")
        canvas.restoreState()

    def _on_page(self, canvas, doc):
        """每页回调：绘制页眉和页脚。"""
        self._draw_page_header(canvas, doc)
        self._draw_page_footer(canvas, doc)
        
        # 注册书签到 PDF 目录大纲
        # 注意：由于 reportlab 的限制，这里只能在页面级别添加书签，
        # 无法精确到段落级别，但对于报告来说已经足够。
        # 我们在 _add_bookmark_and_title 中使用了 <a name="..."/> 锚点，
        # 配合 href="#..." 可以在文档内部跳转。
        # 这里额外添加 PDF 侧边栏大纲支持。
        if hasattr(canvas, 'bookmarkPage'):
            for entry in self._toc_entries:
                # 简单处理：如果当前页是该书签所在的页（近似），则添加到大纲
                # 实际上 reportlab 有更复杂的机制，这里我们主要依赖内部链接
                pass

    # ----- 输出方法 --------------------------------------------------------

    def render(self) -> bytes:
        """渲染 PDF 并返回字节流

        :return: PDF 文件的字节内容
        """
        buf = io.BytesIO()
        try:
            self._build_doc(buf)
            return buf.getvalue()
        finally:
            buf.close()
            self._cleanup_temp_files()

    def save(self, file_path: str) -> str:
        """保存 PDF 报告到文件

        :param file_path: 输出文件路径
        :return: 文件路径
        """
        dir_name = os.path.dirname(file_path)
        if dir_name:
            os.makedirs(dir_name, exist_ok=True)

        try:
            with open(file_path, "wb") as f:
                f.write(self.render())
        finally:
            self._cleanup_temp_files()
        return file_path

    # ----- 内部方法 --------------------------------------------------------

    def _build_doc(self, output):
        """构建 PDF 文档并写入到 output（文件对象或 BytesIO）。

        注意：reportlab 的 ``doc.build()`` 会就地消费 elements 列表，因此这里使用浅拷贝
        以保证 ``render()`` / ``save()`` 可被多次调用。
        """
        doc = SimpleDocTemplate(
            output,
            pagesize=PAGE_SIZE,
            topMargin=MARGIN_TOP,
            bottomMargin=MARGIN_BOTTOM,
            leftMargin=MARGIN_LEFT,
            rightMargin=MARGIN_RIGHT,
            title=self.title,
            author=self.author,
        )
        doc.build(list(self._elements), onFirstPage=self._on_page, onLaterPages=self._on_page)

    def _cleanup_temp_files(self):
        """清理临时图片文件。"""
        for fp in self._temp_files:
            try:
                os.unlink(fp)
            except OSError:
                pass
        self._temp_files.clear()



from collections import OrderedDict
from czsc.core import Direction as Direction, Mark as Mark, Operate as Operate, ZS as ZS
from czsc.signals.tas import update_macd_cache as update_macd_cache
from czsc.traders.base import CZSC as CZSC, CzscTrader as CzscTrader
from czsc.utils import sorted_freqs as sorted_freqs
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements
from loguru import logger as logger

def zdy_bi_end_V230406(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_bi_end_V230407(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_vibrate_V230406(cat: CzscTrader, freq1, freq2, **kwargs) -> OrderedDict: ...
def zdy_stop_loss_V230406(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_take_profit_V230406(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_take_profit_V230407(cat: CzscTrader, **kwargs) -> OrderedDict: ...
def zdy_zs_V230423(c: CZSC, **kwargs): ...
def zdy_macd_bc_V230422(c: CZSC, **kwargs): ...
def zdy_macd_bs1_V230422(c: CZSC, **kwargs): ...
def zdy_zs_space_V230421(c: CZSC, **kwargs): ...
def zdy_macd_dif_V230516(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_dif_V230517(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230518(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230519(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_dif_iqr_V230521(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_macd_V230527(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_dif_V230527(c: CZSC, **kwargs) -> OrderedDict: ...
def zdy_dif_V230528(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240222(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240402(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240406(c: CZSC, **kwargs) -> OrderedDict: ...
def pressure_support_V240530(c: CZSC, **kwargs) -> OrderedDict: ...



from _typeshed import Incomplete
from io import BytesIO

class AliyunOSS:
    auth: Incomplete
    bucket: Incomplete
    def __init__(self, access_key_id: str, access_key_secret: str, endpoint: str, bucket_name: str) -> None: ...
    def upload(self, filepath: str, oss_key: str, replace: bool = False) -> bool: ...
    def download(self, oss_key: str, filepath: str, replace: bool = False) -> bool: ...
    def delete_file(self, oss_key: str) -> bool: ...
    def file_exists(self, oss_key: str) -> bool: ...
    def get_file_stream(self, oss_key: str) -> BytesIO: ...
    def create_folder(self, folder_path: str): ...
    def list_files(self, prefix: str = '', extensions=None): ...
    def batch_upload(self, filepaths: list[str], oss_keys: list[str], replace: bool = False, threads: int = 5): ...
    def batch_download(self, oss_keys: list[str], local_paths: list[str], replace: bool = False, threads: int = 5): ...
    def multipart_upload(self, filepath: str, oss_key: str): ...
    def download_folder(self, oss_folder: str, local_folder: str, threads: int = 5): ...
    def upload_folder(self, local_folder: str, oss_folder: str, replace: bool = False, threads: int = 5): ...



import pandas as pd

def validate_dataframe_columns(df: pd.DataFrame, required_columns: list[str], name: str = 'DataFrame') -> None: ...
def validate_datetime_index(df: pd.DataFrame, name: str = 'DataFrame') -> None: ...
def validate_numeric_column(df: pd.DataFrame, column: str, name: str = 'DataFrame') -> None: ...
def validate_date_range(df: pd.DataFrame, start_date: pd.Timestamp | None = None, end_date: pd.Timestamp | None = None, name: str = 'DataFrame') -> None: ...
def validate_no_duplicates(df: pd.DataFrame, subset: list[str] | None = None, name: str = 'DataFrame') -> None: ...
def validate_weight_data(df: pd.DataFrame) -> None: ...



import pandas as pd
import plotly.graph_objects as go
from _typeshed import Incomplete

COLOR_DRAWDOWN: str
COLOR_RETURN: str
COLOR_ANNO_GRAY: str
COLOR_ANNO_RED: str
COLOR_BORDER: str
COLOR_HEADER_BG: str
QUANTILES_DRAWDOWN: Incomplete
QUANTILES_DRAWDOWN_ANALYSIS: Incomplete
SIGMA_LEVELS: Incomplete
MONTH_LABELS: Incomplete
TemplateType: Incomplete

def figure_to_html(fig: go.Figure, to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def add_year_boundary_lines(fig: go.Figure, dates: pd.DatetimeIndex, row: int | None = None, col: int | None = None, line_color: str = 'red', opacity: float = 0.3, line_dash: str = 'dash') -> None: ...



""" 
权重回测报告生成器

支持 HTML (f-string + plotly) 和 PDF (reportlab + plotly) 两种格式的 WeightBacktest 回测报告生成
"""
import os
from typing import Optional, Dict, Any

import pandas as pd

from rs_czsc import WeightBacktest

from .plotting.backtest import (
    get_performance_metrics_cards,
    plot_backtest_stats,
    plot_long_short_comparison,
    plot_colored_table
)
from .html_report_builder import HtmlReportBuilder
from .pdf_report_builder import PdfReportBuilder


def _validate_input_data(df: pd.DataFrame) -> None:
    """验证输入数据格式
    
    :param df: 输入数据
    :raises ValueError: 当数据格式不正确时
    """
    required_columns = ["dt", "symbol", "weight", "price"]
    missing_columns = [col for col in required_columns if col not in df.columns]
    
    if missing_columns:
        raise ValueError(f"数据缺少必需列: {missing_columns}")
    
    if len(df) == 0:
        raise ValueError("输入数据不能为空")
    
    if df[["weight", "price"]].isna().any().any():
        raise ValueError("权重和价格列不能包含空值")


def _prepare_config(kwargs: dict) -> Dict[str, Any]:
    """准备配置参数
    
    :param kwargs: 用户传入的参数
    :return: 配置字典
    """
    default_config = {
        "fee_rate": 0.0002,
        "digits": 2,
        "weight_type": "ts",
        "yearly_days": 252,
        "n_jobs": 1
    }
    
    config = {**default_config, **kwargs}
    return config


def _build_report_params(df: pd.DataFrame, config: Dict[str, Any]) -> Dict[str, str]:
    """构建报告参数
    
    :param df: 权重数据
    :param config: 配置参数
    :return: 参数字典
    """
    return {
        "日期范围": f"{df['dt'].min().strftime('%Y-%m-%d')} ~ {df['dt'].max().strftime('%Y-%m-%d')}",
        "手续费": f"{config['fee_rate'] * 10000:.2f} BP",
        "小数位": str(config["digits"]),
        "年交易日": str(config["yearly_days"]),
        "标的数": str(df["symbol"].nunique())
    }


def generate_backtest_report(
    df: pd.DataFrame,
    output_path: Optional[str] = None,
    title: str = "权重回测报告",
    **kwargs
) -> str:
    """统一的回测报告生成入口，根据 output_path 的后缀自动选择生成 PDF 或 HTML 报告

    :param df: 包含 dt, symbol, weight, price 列的权重数据
    :param output_path: 报告文件输出路径，根据后缀自动选择格式：
        - .pdf 后缀生成 PDF 报告
        - .html / .htm 后缀生成 HTML 报告
        - 默认为 None，生成当前目录下的 backtest_report.html（HTML 格式）
    :param title: 报告标题
    :param kwargs: 回测参数和显示控制
        - fee_rate: float, 单边交易成本，默认 0.0002
        - digits: int, 权重小数位数，默认 2
        - weight_type: str, 权重类型，默认 'ts'
        - yearly_days: int, 年交易日天数，默认 252
        - n_jobs: int, 并行进程数，默认 1
    :return: 报告文件路径
    :raises ValueError: 当输入数据格式不正确时或文件后缀不支持时
    """
    if output_path is None:
        output_path = os.path.join(os.getcwd(), "backtest_report.html")

    ext = os.path.splitext(output_path)[1].lower()

    if ext == ".pdf":
        return generate_pdf_backtest_report(df, output_path=output_path, title=title, **kwargs)
    elif ext in (".html", ".htm"):
        return generate_html_backtest_report(df, output_path=output_path, title=title, **kwargs)
    else:
        raise ValueError(f"不支持的文件后缀 '{ext}'，请使用 .html 或 .pdf")


def generate_html_backtest_report(
    df: pd.DataFrame,
    output_path: Optional[str] = None,
    title: str = "权重回测报告",
    **kwargs
) -> str:
    """生成权重回测的 HTML 报告

    :param df: 包含 dt, symbol, weight, price 列的权重数据
    :param output_path: HTML 文件输出路径，默认为当前目录下的 backtest_report.html
    :param title: 报告标题
    :param kwargs: 回测参数和显示控制
        - fee_rate: float, 单边交易成本，默认 0.0002
        - digits: int, 权重小数位数，默认 2
        - weight_type: str, 权重类型，默认 'ts'
        - yearly_days: int, 年交易日天数，默认 252
        - n_jobs: int, 并行进程数，默认 1
    :return: HTML 文件路径
    :raises ValueError: 当输入数据格式不正确时
    """
    # 验证输入数据
    _validate_input_data(df)
    
    # 准备配置
    config = _prepare_config(kwargs)
    if output_path is None:
        output_path = os.path.join(os.getcwd(), "backtest_report.html")
    
    # 创建回测实例
    wb = WeightBacktest(
        dfw=df,
        fee_rate=config["fee_rate"],
        digits=config["digits"],
        weight_type=config["weight_type"],
        yearly_days=config["yearly_days"],
        n_jobs=config["n_jobs"]
    )
    
    # 提取数据
    metrics = get_performance_metrics_cards(wb.stats)
    charts = _generate_charts(wb, df, config)
    
    # 构建报告
    _build_and_save_report(title, df, config, metrics, charts, output_path)
    
    return output_path


def _build_and_save_report(title: str, df: pd.DataFrame, config: Dict[str, Any], 
                           metrics: list, charts: dict, output_path: str) -> None:
    """构建并保存报告
    
    :param title: 报告标题
    :param df: 权重数据
    :param config: 配置参数
    :param metrics: 绩效指标
    :param charts: 图表字典
    :param output_path: 输出路径
    """
    builder = HtmlReportBuilder(title=title)
    
    # 添加头部
    params = _build_report_params(df, config)
    builder.add_header(params, subtitle="基于权重策略的回测分析与绩效评估")
    
    # 添加绩效指标
    builder.add_metrics(metrics)
    
    # 添加图表标签页
    builder.add_chart_tab("回测统计", charts["backtest_stats"], "bi-grid-1x2", active=True)
    builder.add_chart_tab("多空对比", charts["long_short_comparison"], "bi-arrows-collapse")
    
    # 添加图表区域
    builder.add_charts_section()
    
    # 添加页脚
    builder.add_footer()
    
    # 保存文件
    builder.save(output_path)


def _prepare_daily_returns(wb: WeightBacktest) -> pd.DataFrame:
    """准备日收益数据
    
    :param wb: WeightBacktest 对象
    :return: 处理后的日收益数据
    """
    dret = wb.daily_return.copy()
    dret["dt"] = pd.to_datetime(dret["date"])
    dret = dret.set_index("dt").drop(columns=["date"])
    return dret


def _generate_main_charts(dret: pd.DataFrame) -> dict:
    """生成主要图表
    
    :param dret: 日收益数据
    :return: 图表字典
    """
    config = {'responsive': True, 'displayModeBar': True, 'scrollZoom': True}
    fig_stats = plot_backtest_stats(dret, ret_col="total", title="", template="plotly")
    fig_stats.update_layout(height=1000, autosize=True)
    
    return {
        "backtest_stats": fig_stats.to_html(include_plotlyjs=True, full_html=False, config=config)
    }


class LongShortComparisonChart:
    """多空对比图表生成器"""
    
    def __init__(self, df: pd.DataFrame, config: Dict[str, Any]):
        """初始化多空对比图表生成器
        
        :param df: 原始权重数据
        :param config: 配置参数
        """
        self.df = df
        self.config = config
        self._strategy_data = None
        self._backtests = None
    
    def _create_strategy_data(self) -> dict:
        """创建多空策略数据
        
        :return: 策略数据字典
        """
        if self._strategy_data is not None:
            return self._strategy_data
            
        df_base = self.df[["dt", "symbol", "weight", "price"]].copy()
        
        dfl = df_base.copy()
        dfl["weight"] = dfl["weight"].clip(lower=0)  # 只保留多头
        
        dfs = df_base.copy()
        dfs["weight"] = dfs["weight"].clip(upper=0)  # 只保留空头
        
        dfb = df_base.copy()
        dfb['weight'] = 1  # 基准等权
        
        self._strategy_data = {
            "原始策略": df_base,
            "策略多头": dfl,
            "策略空头": dfs,
            "基准等权": dfb
        }
        return self._strategy_data
    
    def _create_backtests(self) -> dict:
        """创建策略回测实例
        
        :return: 回测实例字典
        """
        if self._backtests is not None:
            return self._backtests
            
        strategy_data = self._create_strategy_data()
        wbs = {}
        for name, data in strategy_data.items():
            weight_type = "ts" if name == "基准等权" else self.config["weight_type"]
            wbs[name] = WeightBacktest(
                data, 
                fee_rate=self.config["fee_rate"], 
                digits=self.config["digits"],
                weight_type=weight_type, 
                yearly_days=self.config["yearly_days"]
            )
        self._backtests = wbs
        return wbs
    
    def _aggregate_daily_returns(self) -> pd.DataFrame:
        """汇总日收益数据
        
        :return: 汇总后的日收益数据
        """
        wbs = self._create_backtests()
        dailys = []
        for strategy, wb_obj in wbs.items():
            daily = wb_obj.daily_return.copy()[["date", "total"]]
            daily["strategy"] = strategy
            daily["return"] = daily["total"]
            daily["dt"] = pd.to_datetime(daily["date"])
            dailys.append(daily[["dt", "strategy", "return"]])
        
        dailys = pd.concat(dailys, axis=0)
        return pd.pivot_table(dailys, index="dt", columns="strategy", values="return")
    
    def _aggregate_strategy_stats(self) -> pd.DataFrame:
        """汇总策略统计信息
        
        :return: 统计信息 DataFrame
        """
        wbs = self._create_backtests()
        stats_rows = []
        for strategy, wb_obj in wbs.items():
            stats = {"策略名称": strategy}
            stats.update(wb_obj.stats)
            stats_rows.append(stats)
        return pd.DataFrame(stats_rows)
    
    def generate(self) -> str:
        """生成多空对比图表
        
        :return: 图表 HTML 字符串
        """
        try:
            # 准备数据
            df_dailys = self._aggregate_daily_returns()
            df_stats = self._aggregate_strategy_stats()
            
            # 生成图表
            plot_config = {'responsive': True, 'displayModeBar': True, 'scrollZoom': True}
            fig_ls = plot_long_short_comparison(df_dailys, df_stats, title="多空收益对比", template="plotly")
            fig_ls.update_layout(height=1000, autosize=True)
            
            return fig_ls.to_html(include_plotlyjs=False, full_html=False, config=plot_config)
            
        except Exception as e:
            return f"<div style='padding: 20px; text-align: center; color: red;'>多空对比图生成失败: {str(e)}</div>"


def _generate_long_short_chart(df: pd.DataFrame, config: Dict[str, Any]) -> str:
    """生成多空对比图表
    
    :param df: 原始权重数据
    :param config: 配置参数
    :return: 图表 HTML 字符串
    """
    chart_generator = LongShortComparisonChart(df, config)
    return chart_generator.generate()


def _generate_charts(wb: WeightBacktest, df: pd.DataFrame, config: Dict[str, Any]) -> dict:
    """生成所有图表
    
    :param wb: WeightBacktest 对象
    :param df: 原始权重数据
    :param config: 配置参数
    :return: 字典，包含所有图表的 HTML 片段
    """
    # 准备日收益数据
    dret = _prepare_daily_returns(wb)
    
    # 生成主要图表
    charts = _generate_main_charts(dret)
    
    # 生成多空对比图表
    charts["long_short_comparison"] = _generate_long_short_chart(df, config)
    
    return charts


# ========== PDF 回测报告 ==========

def _create_pdf_drawdown(dret, ret_col="total", benchmark_returns=None):
    """PDF 专用：回撤分析图（独立 Figure，双 Y 轴，不含 make_subplots）

    :param dret: 日收益数据
    :param ret_col: 收益列名
    :param benchmark_returns: 等权基准日收益序列（与 dret 索引对齐）
    :return: Plotly Figure
    """
    import plotly.graph_objects as go
    from plotly.subplots import make_subplots
    from .plotting.common import add_year_boundary_lines

    returns = dret[ret_col].fillna(0).sort_index(ascending=True)
    cum_ret = returns.cumsum()
    drawdown = cum_ret - cum_ret.cummax()

    if benchmark_returns is not None:
        benchmark_returns = pd.Series(benchmark_returns).reindex(cum_ret.index).fillna(0)
        bench_cum_ret = benchmark_returns.cumsum()
    else:
        bench_cum_ret = None

    # 使用 make_subplots 创建双 Y 轴（这是唯一合理使用 secondary_y 的场景）
    fig = make_subplots(specs=[[{"secondary_y": True}]])

    fig.add_trace(go.Scatter(
        x=drawdown.index, y=drawdown * 100,
        fillcolor="rgba(250,128,114,0.4)", line=dict(color="salmon"),
        fill="tozeroy", mode="lines", name="回撤 (%)", opacity=0.6,
    ), secondary_y=False)

    fig.add_trace(go.Scatter(
        x=cum_ret.index, y=cum_ret,
        mode="lines", name="策略累计收益", opacity=0.9,
        line=dict(color="#34a853", width=1.5),
    ), secondary_y=True)

    if bench_cum_ret is not None:
        fig.add_trace(go.Scatter(
            x=bench_cum_ret.index, y=bench_cum_ret,
            mode="lines", name="基准累计收益", opacity=0.9,
            line=dict(color="#1f77b4", width=1.5),
        ), secondary_y=True)

    add_year_boundary_lines(fig, dret.index)

    # 添加回撤分位数标注
    for q in [0.05, 0.1, 0.2]:
        y_val = drawdown.quantile(q)
        fig.add_hline(
            y=y_val * 100, line_dash="dot",
            annotation_text=f"{q * 100:.0f}%: {y_val * 100:.2f}%",
            annotation_position="bottom left",
            line_color="rgba(128,128,128,0.5)", line_width=1,
        )

    fig.update_layout(
        title="回撤分析（策略 vs 等权）" if bench_cum_ret is not None else "回撤分析",
        template="plotly",
        height=400,
        margin=dict(l=60, r=60, b=50, t=50),
        hovermode="x unified",
        legend=dict(orientation="h", yanchor="bottom", y=-0.2, xanchor="center", x=0.5),
    )
    fig.update_yaxes(title_text="回撤 (%)", secondary_y=False)
    fig.update_yaxes(title_text="累计收益", secondary_y=True)
    return fig


def _create_pdf_daily_distribution(dret, ret_col="total"):
    """PDF 专用：日收益分布直方图（独立 Figure）

    :param dret: 日收益数据
    :param ret_col: 收益列名
    :return: Plotly Figure
    """
    import plotly.express as px

    daily_pct = (dret[ret_col] * 100).reset_index()
    daily_pct.columns = ["dt", "日收益"]

    fig = px.histogram(daily_pct, x="日收益", nbins=50, title="日收益分布")
    fig.update_xaxes(title="日收益 (%)")
    fig.update_yaxes(title="频数")

    # 添加 sigma 线
    mean_val = daily_pct["日收益"].mean()
    std_val = daily_pct["日收益"].std()
    for sigma in [-3, -2, -1, 1, 2, 3]:
        val = mean_val + sigma * std_val
        fig.add_vline(
            x=val, line_dash="dot",
            line_color="rgba(128,128,128,0.5)", line_width=1,
            annotation_text=f"{sigma}σ: {val:.2f}%",
            annotation_font=dict(color="red", size=9),
            annotation_position="top",
        )

    fig.update_layout(
        template="plotly",
        height=400,
        margin=dict(l=60, r=40, b=50, t=50),
        bargap=0.1,
    )
    return fig


def _create_pdf_monthly_heatmap(dret, ret_col="total"):
    """PDF 专用：月度收益热力图（独立 Figure）

    :param dret: 日收益数据
    :param ret_col: 收益列名
    :return: Plotly Figure
    """
    import plotly.graph_objects as go

    month_labels = ['1月', '2月', '3月', '4月', '5月', '6月',
                    '7月', '8月', '9月', '10月', '11月', '12月']

    df = dret[[ret_col]].copy()
    df['year'] = df.index.year
    df['month'] = df.index.month

    monthly_ret = df.groupby(['year', 'month'])[ret_col].sum() * 100
    monthly_ret = monthly_ret.reset_index()
    monthly_ret.columns = ['年份', '月份', '月收益']

    pivot = monthly_ret.pivot(index='年份', columns='月份', values='月收益')

    monthly_wr = (monthly_ret['月收益'] > 0).mean()
    yearly_ret = monthly_ret.groupby('年份')['月收益'].sum()
    yearly_wr = (yearly_ret > 0).mean()

    fig = go.Figure(data=go.Heatmap(
        z=pivot.values,
        x=month_labels,
        y=pivot.index,
        colorscale='RdYlGn_r',
        colorbar=dict(title="收益 (%)"),
        text=pivot.values,
        texttemplate="%{text:.2f}%",
        textfont={"size": 10},
        hovertemplate="%{y}<br>%{x}<br>收益: %{z:.2f}%<extra></extra>",
    ))

    fig.update_layout(
        title=f"月度收益热力图（月胜率: {monthly_wr:.1%}，年胜率: {yearly_wr:.1%}）",
        template="plotly",
        height=max(300, len(pivot) * 35 + 120),
        margin=dict(l=60, r=40, b=50, t=60),
        xaxis_title="月份",
        yaxis_title="年份",
    )
    return fig


def _create_long_short_curves_figure(dailys_pivot):
    """为 PDF 创建多空对比累计收益曲线图（不含表格子图）

    :param dailys_pivot: 透视表格式的日收益数据
    :return: Plotly Figure 对象
    """
    import numpy as np
    import plotly.graph_objects as go
    import plotly.express as px
    from .plotting.common import add_year_boundary_lines

    target_volatility = 0.2
    trading_days_per_year = 252

    df_cumsum = dailys_pivot.cumsum()
    colors = px.colors.qualitative.Plotly * 10

    fig = go.Figure()

    for i, col in enumerate(df_cumsum.columns):
        fig.add_trace(go.Scatter(
            x=df_cumsum.index, y=df_cumsum[col],
            name=col, mode="lines", line=dict(color=colors[i]),
        ))

    add_year_boundary_lines(fig, df_cumsum.index)

    fig.update_layout(
        title="多空累计收益曲线对比",
        template="plotly",
        height=400,
        margin=dict(l=60, r=40, b=50, t=50),
        hovermode="x unified",
        xaxis_title="",
        yaxis_title="累计收益",
        legend=dict(orientation="h", yanchor="bottom", y=-0.2, xanchor="center", x=0.5),
    )
    return fig


def _create_long_short_adjusted_figure(dailys_pivot, target_volatility=0.2):
    """为 PDF 创建波动率调整后的多空对比累计收益曲线

    :param dailys_pivot: 透视表格式的日收益数据
    :param target_volatility: 目标年化波动率
    :return: Plotly Figure 对象
    """
    import numpy as np
    import plotly.graph_objects as go
    import plotly.express as px
    from .plotting.common import add_year_boundary_lines

    trading_days_per_year = 252
    adjusted_returns = pd.DataFrame(index=dailys_pivot.index)

    for col in dailys_pivot.columns:
        daily_ret = dailys_pivot[col]
        annual_vol = daily_ret.std() * np.sqrt(trading_days_per_year)
        factor = target_volatility / annual_vol if annual_vol > 0 else 1.0
        adjusted_returns[col] = daily_ret * factor

    df_adj_cumsum = adjusted_returns.cumsum()
    colors = px.colors.qualitative.Plotly * 10

    fig = go.Figure()
    for i, col in enumerate(df_adj_cumsum.columns):
        fig.add_trace(go.Scatter(
            x=df_adj_cumsum.index, y=df_adj_cumsum[col],
            name=col, mode="lines", line=dict(color=colors[i]),
        ))

    add_year_boundary_lines(fig, df_adj_cumsum.index)

    fig.update_layout(
        title=f"波动率调整后收益对比（目标波动率: {target_volatility:.0%}）",
        template="plotly",
        height=400,
        margin=dict(l=60, r=40, b=50, t=50),
        hovermode="x unified",
        xaxis_title="",
        yaxis_title="调整后累计收益",
        legend=dict(orientation="h", yanchor="bottom", y=-0.2, xanchor="center", x=0.5),
    )
    return fig


def _create_long_short_table_figure(stats_df):
    """为 PDF 创建多空绩效对比表格图

    :param stats_df: 绩效指标 DataFrame
    :return: Plotly Figure 对象
    """
    key_cols = [
        "策略名称", "年化", "夏普", "卡玛", "最大回撤",
        "年化波动率", "交易胜率", "单笔收益", "持仓K线数", "多头占比", "空头占比",
    ]
    available_cols = [c for c in key_cols if c in stats_df.columns]
    table_df = stats_df[available_cols].copy()
    if "策略名称" in table_df.columns:
        table_df = table_df.set_index("策略名称")

    fig = plot_colored_table(
        table_df, title="绩效指标对比", template="plotly", float_fmt=".2f",
        good_high_columns=["年化", "夏普", "卡玛", "交易胜率", "单笔收益"],
    )
    fig.update_layout(height=250, margin=dict(l=40, r=40, b=20, t=50))
    return fig


def _build_stats_summary_table(wb) -> pd.DataFrame:
    """从 WeightBacktest 构建详细的统计摘要表

    :param wb: WeightBacktest 对象
    :return: 统计摘要 DataFrame（两列：指标名称、指标值）
    """
    stats = wb.stats
    rows = []

    # 收益相关
    rows.append(("年化收益率", f"{stats.get('年化', 0):.4f}"))
    rows.append(("单笔收益(BP)", f"{stats.get('单笔收益', 0):.2f}"))
    rows.append(("交易胜率", f"{stats.get('交易胜率', 0):.4f}"))

    # 风险相关
    rows.append(("最大回撤", f"{stats.get('最大回撤', 0):.4f}"))
    rows.append(("年化波动率", f"{stats.get('年化波动率', 0):.4f}"))
    rows.append(("夏普比率", f"{stats.get('夏普', 0):.2f}"))
    rows.append(("卡玛比率", f"{stats.get('卡玛', 0):.2f}"))

    # 持仓相关
    rows.append(("持仓K线数", f"{stats.get('持仓K线数', 0):.0f}"))
    rows.append(("多头占比", f"{stats.get('多头占比', 0):.4f}"))
    rows.append(("空头占比", f"{stats.get('空头占比', 0):.4f}"))

    # 其他可能存在的指标
    for key in ["日胜率", "盈亏比", "最大连续亏损"]:
        if key in stats:
            rows.append((key, f"{stats[key]:.4f}" if isinstance(stats[key], float) else str(stats[key])))

    df = pd.DataFrame(rows, columns=["指标", "数值"])
    return df


def _generate_pdf_figures(wb, df: pd.DataFrame, config: dict) -> dict:
    """生成 PDF 报告所需的 Plotly Figure 对象（每个图表独立，不使用 make_subplots 组合）

    :param wb: WeightBacktest 对象
    :param df: 原始权重数据
    :param config: 配置参数
    :return: 有序字典，key 为图表名称，value 为 Plotly Figure 对象
    """
    dret = _prepare_daily_returns(wb)

    figures = {}

    # 1. 回撤分析（含策略与等权累计收益）
    df_bench = df[["dt", "symbol", "weight", "price"]].copy()
    df_bench["weight"] = 1
    wb_bench = WeightBacktest(
        dfw=df_bench,
        fee_rate=config["fee_rate"],
        digits=config["digits"],
        weight_type="ts",
        yearly_days=config["yearly_days"],
        n_jobs=config["n_jobs"],
    )
    bench_daily = wb_bench.daily_return.copy()
    bench_daily["dt"] = pd.to_datetime(bench_daily["date"])
    bench_daily = bench_daily.set_index("dt").sort_index(ascending=True)

    fig_dd = _create_pdf_drawdown(
        dret,
        ret_col="total",
        benchmark_returns=bench_daily["total"],
    )
    figures["drawdown_analysis"] = fig_dd

    # 2. 日收益分布
    fig_dist = _create_pdf_daily_distribution(dret, ret_col="total")
    figures["daily_distribution"] = fig_dist

    # 3. 月度收益热力图
    fig_heatmap = _create_pdf_monthly_heatmap(dret, ret_col="total")
    figures["monthly_heatmap"] = fig_heatmap

    # 5. 多空对比图
    try:
        ls_chart = LongShortComparisonChart(df, config)
        df_dailys = ls_chart._aggregate_daily_returns()
        df_stats = ls_chart._aggregate_strategy_stats()

        # 多空累计收益曲线
        fig_ls = _create_long_short_curves_figure(df_dailys)
        figures["long_short_curves"] = fig_ls

        # 波动率调整后收益
        fig_ls_adj = _create_long_short_adjusted_figure(df_dailys)
        figures["long_short_adjusted"] = fig_ls_adj

        # 绩效对比表
        fig_ls_table = _create_long_short_table_figure(df_stats)
        figures["long_short_table"] = fig_ls_table
    except Exception:
        pass

    return figures


def _build_and_save_pdf_report(title: str, df: pd.DataFrame, config: dict,
                               metrics: list, figures: dict, output_path: str,
                               wb=None) -> None:
    """构建并保存 PDF 报告

    PDF 专属布局策略：
    - 第1页：标题 + 参数 + 目录 + 核心绩效指标卡片
    - 第2页：回撤分析（含策略 vs 等权收益，适配整页）
    - 第3页：日收益分布（适配整页）
    - 第4页：月度收益热力图（适配整页）
    - 第5页起：多空对比各图表

    :param title: 报告标题
    :param df: 权重数据
    :param config: 配置参数
    :param metrics: 绩效指标
    :param figures: Plotly Figure 字典
    :param output_path: 输出路径
    :param wb: WeightBacktest 对象（保留参数兼容）
    """
    builder = PdfReportBuilder(title=title)

    # ===== 1. 头部 + 指标卡片 =====
    params = _build_report_params(df, config)
    builder.add_header(params, subtitle="基于权重策略的回测分析与绩效评估")
    builder.add_metrics(metrics)

    # ===== 2. 各图表：每个独立一页，fit_page 填充 =====
    chart_sequence = [
        ("drawdown_analysis", "回撤分析"),
        ("daily_distribution", "日收益分布"),
        ("monthly_heatmap", "月度收益热力图"),
        ("long_short_curves", "多空累计收益对比"),
        ("long_short_adjusted", "波动率调整后收益对比"),
        ("long_short_table", "多空绩效指标对比"),
    ]

    for key, chart_title in chart_sequence:
        if key in figures:
            builder.add_page_break()
            is_table = key == "long_short_table"
            builder.add_chart(
                figures[key],
                title=chart_title,
                fit_page=not is_table,
                height=5.0 if is_table else None,
                aspect_ratio=0.6 if key == "monthly_heatmap" else 0.55,
            )

    # ===== 3. 在头部后插入目录（不分页，和核心绩效指标同页显示） =====
    builder.insert_toc_after_header(title="目 录", add_page_break=False)

    # ===== 4. 页脚 =====
    builder.add_footer()

    # ===== 5. 保存 =====
    builder.save(output_path)


def generate_pdf_backtest_report(
    df: pd.DataFrame,
    output_path: Optional[str] = None,
    title: str = "权重回测报告",
    **kwargs
) -> str:
    """生成权重回测的 PDF 报告

    :param df: 包含 dt, symbol, weight, price 列的权重数据
    :param output_path: PDF 文件输出路径，默认为当前目录下的 backtest_report.pdf
    :param title: 报告标题
    :param kwargs: 回测参数和显示控制
        - fee_rate: float, 单边交易成本，默认 0.0002
        - digits: int, 权重小数位数，默认 2
        - weight_type: str, 权重类型，默认 'ts'
        - yearly_days: int, 年交易日天数，默认 252
        - n_jobs: int, 并行进程数，默认 1
    :return: PDF 文件路径
    :raises ValueError: 当输入数据格式不正确时
    """
    # 验证输入数据
    _validate_input_data(df)

    # 准备配置
    config = _prepare_config(kwargs)
    if output_path is None:
        output_path = os.path.join(os.getcwd(), "backtest_report.pdf")

    # 创建回测实例
    wb = WeightBacktest(
        dfw=df,
        fee_rate=config["fee_rate"],
        digits=config["digits"],
        weight_type=config["weight_type"],
        yearly_days=config["yearly_days"],
        n_jobs=config["n_jobs"]
    )

    # 提取数据
    metrics = get_performance_metrics_cards(wb.stats)
    figures = _generate_pdf_figures(wb, df, config)

    # 构建报告
    _build_and_save_pdf_report(title, df, config, metrics, figures, output_path, wb=wb)

    return output_path



import numpy as np
import pandas as pd
from _typeshed import Incomplete

def SMA(close: np.array, timeperiod: int = 5): ...
def WMA(close: np.array, timeperiod: int = 5): ...
def EMA(close: np.array, timeperiod: int = 5): ...
def MACD(real: np.array, fastperiod: int = 12, slowperiod: int = 26, signalperiod: int = 9): ...
def KDJ(close: np.array, high: np.array, low: np.array): ...
def RSQ(close: [np.array, list]) -> float: ...
def PLUS_DI(high, low, close, timeperiod: int = 14): ...
def MINUS_DI(high, low, close, timeperiod: int = 14): ...
def ATR(high, low, close, timeperiod: int = 14): ...
def MFI(high, low, close, volume, timeperiod: int = 14): ...
def CCI(high, low, close, timeperiod: int = 14): ...
def LINEARREG_ANGLE(real, timeperiod: int = 14): ...
def DOUBLE_SMA_LS(series: pd.Series, n: int = 5, m: int = 20, **kwargs): ...
def BOLL_LS(series: pd.Series, n: int = 5, s: float = 0.1, **kwargs): ...
def SMA_MIN_MAX_SCALE(series: pd.Series, timeperiod: int = 5, window: int = 5, **kwargs): ...
def RS_VOLATILITY(df: pd.DataFrame, timeperiod: int = 30, **kwargs): ...
def PK_VOLATILITY(df: pd.DataFrame, timeperiod: int = 30, **kwargs): ...
def SNR(real: pd.Series, timeperiod: int = 14, **kwargs): ...

SMA: Incomplete
EMA: Incomplete
MACD: Incomplete
PPO: Incomplete
ATR: Incomplete
PLUS_DI: Incomplete
MINUS_DI: Incomplete
MFI: Incomplete
CCI: Incomplete
BOLL: Incomplete
RSI: Incomplete
ADX: Incomplete
ADXR: Incomplete
AROON: Incomplete
AROONOSC: Incomplete
ROCR: Incomplete
ROCR100: Incomplete
TRIX: Incomplete
ULTOSC: Incomplete
WILLR: Incomplete
LINEARREG: Incomplete
LINEARREG_ANGLE: Incomplete
LINEARREG_INTERCEPT: Incomplete
LINEARREG_SLOPE: Incomplete
KAMA: Incomplete
STOCH: Incomplete
STOCHF: Incomplete
STOCHRSI: Incomplete
T3: Incomplete
TEMA: Incomplete
TRIMA: Incomplete
WMA: Incomplete
BBANDS: Incomplete
DEMA: Incomplete
HT_TRENDLINE: Incomplete
BOP: Incomplete
CMO: Incomplete
DX: Incomplete
BETA: Incomplete

def CHOP(high, low, close, **kwargs): ...
def rolling_polyfit(real: pd.Series, window: int = 20, degree: int = 1): ...
def rolling_auto_corr(real: pd.Series, window: int = 20, lag: int = 1): ...
def rolling_ptp(real: pd.Series, window: int = 20): ...
def rolling_skew(real: pd.Series, window: int = 20): ...
def rolling_kurt(real: pd.Series, window: int = 20): ...
def rolling_corr(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_cov(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_beta(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_alpha(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_rsq(x: pd.Series, window: int = 20): ...
def rolling_argmax(x: pd.Series, window: int = 20): ...
def rolling_argmin(x: pd.Series, window: int = 20): ...
def rolling_ir(x: pd.Series, window: int = 20): ...
def rolling_zscore(x: pd.Series, window: int = 20): ...
def rolling_rank(x: pd.Series, window: int = 20): ...
def rolling_max(x: pd.Series, window: int = 20): ...
def rolling_min(x: pd.Series, window: int = 20): ...
def rolling_mdd(x: pd.Series, window: int = 20): ...
def rolling_rank_sub(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_rank_div(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_rank_mul(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_rank_sum(x: pd.Series, y: pd.Series, window: int = 20): ...
def rolling_vwap(close: pd.Series, volume: pd.Series, window: int = 20): ...
def rolling_obv(close: pd.Series, volume: pd.Series, window: int = 200): ...
def rolling_pvt(close: pd.Series, volume: pd.Series, window: int = 20): ...
def rolling_pvi(close: pd.Series, volume: pd.Series, window: int = 20): ...
def rolling_std(real: pd.Series, window: int = 20): ...
def ultimate_smoother(price, period: int = 7): ...
def sigmoid(x): ...
def log_return(x): ...



"""
数据格式转换工具

提取的通用数据格式转换逻辑
"""

import pandas as pd
from typing import List, Dict, Any, Optional


def to_standard_kline_format(
    df: pd.DataFrame,
    dt_col: str = 'dt',
    open_col: str = 'open',
    high_col: str = 'high',
    low_col: str = 'low',
    close_col: str = 'close',
    volume_col: str = 'vol'
) -> pd.DataFrame:
    """将DataFrame转换为标准K线格式

    标准K线格式包含: dt, open, close, high, low, vol

    :param df: 输入DataFrame
    :param dt_col: 日期时间列名
    :param open_col: 开盘价列名
    :param high_col: 最高价列名
    :param low_col: 最低价列名
    :param close_col: 收盘价列名
    :param volume_col: 成交量列名
    :return: 标准格式的DataFrame
    """
    result = pd.DataFrame()
    
    # 标准化列名
    column_mapping = {
        dt_col: 'dt',
        open_col: 'open',
        high_col: 'high',
        low_col: 'low',
        close_col: 'close',
        volume_col: 'vol'
    }
    
    for old_col, new_col in column_mapping.items():
        if old_col in df.columns:
            result[new_col] = df[old_col]
        else:
            import warnings
            warnings.warn(f"列 '{old_col}' 不存在于输入DataFrame中")
    
    # 确保dt列是datetime类型
    if 'dt' in result.columns:
        result['dt'] = pd.to_datetime(result['dt'])
    
    return result


def pivot_weight_data(
    dfw: pd.DataFrame,
    fill_value: float = 0.0
) -> pd.DataFrame:
    """将权重数据转换为透视表格式

    :param dfw: 权重数据，包含 dt, symbol, weight 列
    :param fill_value: 填充缺失值
    :return: 透视后的DataFrame，index为dt，columns为symbol
    """
    df = dfw.copy()
    df['dt'] = pd.to_datetime(df['dt'])
    
    pivot_df = pd.pivot_table(
        df,
        index='dt',
        columns='symbol',
        values='weight',
        aggfunc='sum'
    )
    
    if fill_value is not None:
        pivot_df = pivot_df.fillna(fill_value)
    
    return pivot_df


def resample_to_period(
    df: pd.DataFrame,
    period: str,
    agg_dict: Optional[Dict[str, str]] = None
) -> pd.DataFrame:
    """将数据重采样到指定周期

    :param df: 输入DataFrame，必须有DatetimeIndex
    :param period: 周期字符串，如 'D' (日), 'W' (周), 'M' (月)
    :param agg_dict: 聚合函数字典，键为列名，值为聚合函数
    :return: 重采样后的DataFrame
    """
    if not isinstance(df.index, pd.DatetimeIndex):
        raise ValueError("DataFrame必须有DatetimeIndex才能进行重采样")
    
    if agg_dict is None:
        # 默认聚合方式
        agg_dict = {col: 'sum' for col in df.columns}
    
    return df.resample(period).agg(agg_dict)


def normalize_symbol(symbol: str) -> str:
    """标准化品种代码

    :param symbol: 品种代码
    :return: 标准化后的品种代码
    """
    # 移除空格
    symbol = symbol.strip()
    # 转换为大写
    symbol = symbol.upper()
    return symbol


def convert_dict_to_dataframe(
    data: List[Dict[str, Any]],
    sort_by: Optional[str] = None
) -> pd.DataFrame:
    """将字典列表转换为DataFrame

    :param data: 字典列表
    :param sort_by: 排序列名
    :return: DataFrame
    """
    df = pd.DataFrame(data)
    
    if sort_by and sort_by in df.columns:
        df = df.sort_values(by=sort_by)
    
    return df


def ensure_datetime_column(
    df: pd.DataFrame,
    column: str,
    inplace: bool = False
) -> pd.DataFrame:
    """确保指定列是datetime类型

    :param df: DataFrame
    :param column: 列名
    :param inplace: 是否在原DataFrame上修改
    :return: DataFrame
    """
    if not inplace:
        df = df.copy()
    
    if column in df.columns:
        df[column] = pd.to_datetime(df[column])
    
    return df


def flatten_multiindex_columns(
    df: pd.DataFrame,
    sep: str = '_'
) -> pd.DataFrame:
    """扁平化MultiIndex列

    :param df: 包含MultiIndex列的DataFrame
    :param sep: 连接符
    :return: 扁平化后的DataFrame
    """
    if isinstance(df.columns, pd.MultiIndex):
        df.columns = [sep.join(map(str, col)).strip() for col in df.columns.values]
    
    return df



# coding: utf-8

import os
import pickle
import json
import zipfile


def dill_dump(data, file):
    import dill
    with open(file, "wb") as f:
        dill.dump(data, f)


def dill_load(file):
    import dill
    with open(file, "rb") as f:
        data = dill.load(f)
    return data


def save_pkl(data, file):
    with open(file, "wb") as f:
        pickle.dump(data, f)


def read_pkl(file):
    with open(file, "rb") as f:
        data = pickle.load(f)
    return data


def save_json(data, file):
    with open(file, "w", encoding='utf-8') as f:
        json.dump(data, f, ensure_ascii=False, indent=2)


def read_json(file):
    with open(file, "r", encoding='utf-8') as f:
        data = json.load(f)
    return data


def make_zip(source_dir: str, file_zip: str) -> None:
    """打包目录为zip文件

    :param source_dir: 文件夹路径
    :param file_zip: 输出 zip 文件名称
    :return: None
    """
    assert os.path.isdir(source_dir)

    f = zipfile.ZipFile(file_zip, 'w', compression=zipfile.ZIP_DEFLATED)
    for parent, _, filenames in os.walk(source_dir):
        for filename in filenames:
            file = os.path.join(parent, filename)
            f.write(file, file.replace(source_dir, ""))
    f.close()
    print(f"compress {source_dir} into {file_zip} success.")



"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/3/10 11:21
describe: 缠论分型、笔的识别
"""
import os
import webbrowser
from typing import List
from collections import OrderedDict
from czsc.py.enum import Mark, Direction
from czsc.py.objects import BI, FX, RawBar, NewBar
from czsc import envs


def remove_include(k1: NewBar, k2: NewBar, k3: RawBar):
    """去除包含关系：输入三根k线，其中k1和k2为没有包含关系的K线，k3为原始K线

    处理逻辑如下：

    1. 首先，通过比较k1和k2的高点(high)的大小关系来确定direction的值。如果k1的高点小于k2的高点，
       则设定direction为Up；如果k1的高点大于k2的高点，则设定direction为Down；如果k1和k2的高点相等，
       则创建一个新的K线k4，与k3具有相同的属性，并返回False和k4。

    2. 接下来，判断k2和k3之间是否存在包含关系。如果存在，则根据direction的值进行处理。
        - 如果direction为Up，则选择k2和k3中的较大高点作为新K线k4的高点，较大低点作为低点，较大高点所在的时间戳(dt)作为k4的时间戳。
        - 如果direction为Down，则选择k2和k3中的较小高点作为新K线k4的高点，较小低点作为低点，较小低点所在的时间戳(dt)作为k4的时间戳。
        - 如果direction的值不是Up也不是Down，则抛出ValueError异常。

    3. 根据上述处理得到的高点、低点、开盘价(open_)、收盘价(close)，计算新K线k4的成交量(vol)和成交金额(amount)，
       并将k2中除了与k3时间戳相同的元素之外的其他元素与k3一起作为k4的元素列表(elements)。

    4. 返回一个布尔值和新的K线k4。如果k2和k3之间存在包含关系，则返回True和k4；否则返回False和k4，其中k4与k3具有相同的属性。
    """
    if k1.high < k2.high:
        direction = Direction.Up
    elif k1.high > k2.high:
        direction = Direction.Down
    else:
        k4 = NewBar(symbol=k3.symbol, id=k3.id, freq=k3.freq, dt=k3.dt, open=k3.open,
                    close=k3.close, high=k3.high, low=k3.low, vol=k3.vol, amount=k3.amount, elements=[k3])
        return False, k4

    # 判断 k2 和 k3 之间是否存在包含关系，有则处理
    if (k2.high <= k3.high and k2.low >= k3.low) or (k2.high >= k3.high and k2.low <= k3.low):

        if direction == Direction.Up:
            high = max(k2.high, k3.high)
            low = max(k2.low, k3.low)
            dt = k2.dt if k2.high > k3.high else k3.dt

        elif direction == Direction.Down:
            high = min(k2.high, k3.high)
            low = min(k2.low, k3.low)
            dt = k2.dt if k2.low < k3.low else k3.dt

        else:
            raise ValueError

        open_, close = (high, low) if k3.open > k3.close else (low, high)
        vol = k2.vol + k3.vol
        amount = k2.amount + k3.amount

        # 这里有一个隐藏Bug，len(k2.elements) 在一些及其特殊的场景下会有超大的数量，具体问题还没找到；
        # 临时解决方案是直接限定len(k2.elements)<=100
        elements = [x for x in k2.elements[:100] if x.dt != k3.dt] + [k3]
        k4 = NewBar(symbol=k3.symbol, id=k2.id, freq=k2.freq, dt=dt, open=open_,
                    close=close, high=high, low=low, vol=vol, amount=amount, elements=elements)
        return True, k4

    else:
        k4 = NewBar(symbol=k3.symbol, id=k3.id, freq=k3.freq, dt=k3.dt, open=k3.open,
                    close=k3.close, high=k3.high, low=k3.low, vol=k3.vol, amount=k3.amount, elements=[k3])
        return False, k4


def check_fx(k1: NewBar, k2: NewBar, k3: NewBar):
    """查找分型

    函数计算逻辑：

    1. 如果第二个`NewBar`对象的最高价和最低价都高于第一个和第三个`NewBar`对象的对应价格，那么它被认为是顶分型（G）。
       在这种情况下，函数会创建一个新的`FX`对象，其标记为`Mark.G`，并将其赋值给`fx`。

    2. 如果第二个`NewBar`对象的最高价和最低价都低于第一个和第三个`NewBar`对象的对应价格，那么它被认为是底分型（D）。
       在这种情况下，函数会创建一个新的`FX`对象，其标记为`Mark.D`，并将其赋值给`fx`。

    3. 函数最后返回`fx`，如果没有找到分型，`fx`将为`None`。

    :param k1: 第一个`NewBar`对象
    :param k2: 第二个`NewBar`对象
    :param k3: 第三个`NewBar`对象
    :return: `FX`对象或`None`
    """
    fx = None
    if k1.high < k2.high > k3.high and k1.low < k2.low > k3.low:
        fx = FX(symbol=k1.symbol, dt=k2.dt, mark=Mark.G, high=k2.high,
                low=k2.low, fx=k2.high, elements=[k1, k2, k3])

    if k1.low > k2.low < k3.low and k1.high > k2.high < k3.high:
        fx = FX(symbol=k1.symbol, dt=k2.dt, mark=Mark.D, high=k2.high,
                low=k2.low, fx=k2.low, elements=[k1, k2, k3])

    return fx


def check_fxs(bars: List[NewBar]) -> List[FX]:
    """输入一串无包含关系K线，查找其中所有分型

    函数的主要步骤：

    1. 创建一个空列表`fxs`用于存储找到的分型。
    2. 遍历`bars`列表中的每个元素（除了第一个和最后一个），并对每三个连续的`NewBar`对象调用`check_fx`函数。
    3. 如果`check_fx`函数返回一个`FX`对象，检查它的标记是否与`fxs`列表中最后一个`FX`对象的标记相同。如果相同，记录一个错误日志。
       如果不同，将这个`FX`对象添加到`fxs`列表中。
    4. 最后返回`fxs`列表，它包含了`bars`列表中所有找到的分型。

    这个函数的主要目的是找出`bars`列表中所有的顶分型和底分型，并确保它们是交替出现的。如果发现连续的两个分型标记相同，它会记录一个错误日志。

    :param bars: 无包含关系K线列表
    :return: 分型列表
    """
    fxs = []
    for i in range(1, len(bars) - 1):
        fx = check_fx(bars[i - 1], bars[i], bars[i + 1])
        if isinstance(fx, FX):
            # 默认情况下，fxs本身是顶底交替的，但是对于一些特殊情况下不是这样; 临时强制要求fxs序列顶底交替
            if len(fxs) >= 2 and fx.mark == fxs[-1].mark:
                import warnings
                warnings.warn(f"check_fxs错误: {bars[i].dt}，{fx.mark}，{fxs[-1].mark}")
            else:
                fxs.append(fx)
    return fxs


def check_bi(bars: List[NewBar], **kwargs):
    """输入一串无包含关系K线，查找其中的一笔

    :param bars: 无包含关系K线列表
    :return:
    """
    min_bi_len = envs.get_min_bi_len()
    fxs = check_fxs(bars)
    if len(fxs) < 2:
        return None, bars

    fx_a = fxs[0]
    if fx_a.mark == Mark.D:
        direction = Direction.Up
        fxs_b = (x for x in fxs if x.mark == Mark.G and x.dt > fx_a.dt and x.fx > fx_a.fx)
        fx_b = max(fxs_b, key=lambda fx: fx.high, default=None)

    elif fx_a.mark == Mark.G:
        direction = Direction.Down
        fxs_b = (x for x in fxs if x.mark == Mark.D and x.dt > fx_a.dt and x.fx < fx_a.fx)
        fx_b = min(fxs_b, key=lambda fx: fx.low, default=None)

    else:
        raise ValueError

    if fx_b is None:
        return None, bars

    bars_a = [x for x in bars if fx_a.elements[0].dt <= x.dt <= fx_b.elements[2].dt]
    bars_b = [x for x in bars if x.dt >= fx_b.elements[0].dt]

    # 判断fx_a和fx_b价格区间是否存在包含关系
    ab_include = (fx_a.high > fx_b.high and fx_a.low < fx_b.low) or (fx_a.high < fx_b.high and fx_a.low > fx_b.low)

    # 成笔的条件：1）顶底分型之间没有包含关系；2）笔长度大于等于min_bi_len
    if (not ab_include) and (len(bars_a) >= min_bi_len):
        fxs_ = [x for x in fxs if fx_a.elements[0].dt <= x.dt <= fx_b.elements[2].dt]
        bi = BI(symbol=fx_a.symbol, fx_a=fx_a, fx_b=fx_b, fxs=fxs_, direction=direction, bars=bars_a)
        return bi, bars_b
    else:
        return None, bars


class CZSC:
    def __init__(self,
                 bars: List[RawBar],
                 get_signals = None,
                 max_bi_num=envs.get_max_bi_num(),
                 ):
        """

        :param bars: K线数据
        :param max_bi_num: 最大允许保留的笔数量
        :param get_signals: 自定义的信号计算函数
        """
        self.verbose = envs.get_verbose()
        self.max_bi_num = max_bi_num
        self.bars_raw: List[RawBar] = []  # 原始K线序列
        self.bars_ubi: List[NewBar] = []  # 未完成笔的无包含K线序列
        self.bi_list: List[BI] = []
        self.symbol = bars[0].symbol
        self.freq = bars[0].freq
        self.get_signals = get_signals
        self.signals = None
        # cache 是信号计算过程的缓存容器，需要信号计算函数自行维护
        self.cache = OrderedDict()

        for bar in bars:
            self.update(bar)

    def __repr__(self):
        return "<CZSC~{}~{}>".format(self.symbol, self.freq.value)

    def __update_bi(self):
        bars_ubi = self.bars_ubi
        if len(bars_ubi) < 3:
            return

        # 查找笔
        if not self.bi_list:
            # 第一笔的查找
            fxs = check_fxs(bars_ubi)
            if not fxs:
                return

            fx_a = fxs[0]
            fxs_a = [x for x in fxs if x.mark == fx_a.mark]
            for fx in fxs_a:
                if (fx_a.mark == Mark.D and fx.low <= fx_a.low) \
                        or (fx_a.mark == Mark.G and fx.high >= fx_a.high):
                    fx_a = fx
            bars_ubi = [x for x in bars_ubi if x.dt >= fx_a.elements[0].dt]

            bi, bars_ubi_ = check_bi(bars_ubi)
            if isinstance(bi, BI):
                self.bi_list.append(bi)
            self.bars_ubi = bars_ubi_
            return

        if self.verbose and len(bars_ubi) > 100:
            from loguru import logger
            logger.info(f"{self.symbol} - {self.freq} - {bars_ubi[-1].dt} 未完成笔延伸数量: {len(bars_ubi)}")

        bi, bars_ubi_ = check_bi(bars_ubi)
        self.bars_ubi = bars_ubi_
        if isinstance(bi, BI):
            self.bi_list.append(bi)

        # 后处理：如果当前笔被破坏，将当前笔的bars与bars_ubi进行合并，并丢弃
        last_bi = self.bi_list[-1]
        bars_ubi = self.bars_ubi
        if (last_bi.direction == Direction.Up and bars_ubi[-1].high > last_bi.high) \
                or (last_bi.direction == Direction.Down and bars_ubi[-1].low < last_bi.low):
            # 当前笔被破坏，将当前笔的bars与bars_ubi进行合并，并丢弃，这里容易出错，多一根K线就可能导致错误
            # 必须是 -2，因为最后一根无包含K线有可能是未完成的
            self.bars_ubi = last_bi.bars[:-2] + [x for x in bars_ubi if x.dt >= last_bi.bars[-2].dt]
            self.bi_list.pop(-1)

    def update(self, bar: RawBar):
        """更新分析结果

        :param bar: 单根K线对象
        """
        # 更新K线序列
        if not self.bars_raw or bar.dt != self.bars_raw[-1].dt:
            self.bars_raw.append(bar)
            last_bars = [bar]
        else:
            # 当前 bar 是上一根 bar 的时间延伸
            self.bars_raw[-1] = bar
            last_bars = self.bars_ubi.pop(-1).raw_bars
            assert bar.dt == last_bars[-1].dt, f"{bar.dt} != {last_bars[-1].dt}，时间错位"
            last_bars[-1] = bar

        # 去除包含关系
        bars_ubi = self.bars_ubi
        for bar in last_bars:
            if len(bars_ubi) < 2:
                bars_ubi.append(NewBar(symbol=bar.symbol, id=bar.id, freq=bar.freq, dt=bar.dt,
                                       open=bar.open, close=bar.close, amount=bar.amount,
                                       high=bar.high, low=bar.low, vol=bar.vol, elements=[bar]))
            else:
                k1, k2 = bars_ubi[-2:]
                has_include, k3 = remove_include(k1, k2, bar)
                if has_include:
                    bars_ubi[-1] = k3
                else:
                    bars_ubi.append(k3)
        self.bars_ubi = bars_ubi

        # 更新笔
        self.__update_bi()

        # 根据最大笔数量限制完成 bi_list, bars_raw 序列的数量控制
        self.bi_list = self.bi_list[-self.max_bi_num:]
        if self.bi_list:
            sdt = self.bi_list[0].fx_a.elements[0].dt
            s_index = 0
            for i, bar in enumerate(self.bars_raw):
                if bar.dt >= sdt:
                    s_index = i
                    break
            self.bars_raw = self.bars_raw[s_index:]

        # 如果有信号计算函数，则进行信号计算
        self.signals = self.get_signals(c=self) if self.get_signals else OrderedDict()

    def to_echarts(self, width: str = "1400px", height: str = '580px', bs=[]):
        """绘制K线分析图

        :param width: 宽
        :param height: 高
        :param bs: 交易标记，默认为空
        :return:
        """
        from czsc.utils.echarts_plot import kline_pro

        kline = [x.__dict__ for x in self.bars_raw]
        if len(self.bi_list) > 0:
            bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in self.bi_list] + \
                 [{'dt': self.bi_list[-1].fx_b.dt, "bi": self.bi_list[-1].fx_b.fx}]
            fx = [{'dt': x.dt, "fx": x.fx} for x in self.fx_list]
        else:
            bi = []
            fx = []
        chart = kline_pro(kline, bi=bi, fx=fx, width=width, height=height, bs=bs,
                          title="{}-{}".format(self.symbol, self.freq.value))
        return chart

    def to_plotly(self):
        """使用 plotly 绘制K线分析图"""
        import pandas as pd
        from czsc.utils.plotting.kline import KlineChart

        bi_list = self.bi_list
        df = pd.DataFrame(self.bars_raw)
        kline = KlineChart(n_rows=3, title="{}-{}".format(self.symbol, self.freq.value))
        kline.add_kline(df, name="")
        kline.add_sma(df, ma_seq=(5, 10, 21), row=1, visible=True, line_width=1.2)
        kline.add_sma(df, ma_seq=(34, 55, 89, 144), row=1, visible=False, line_width=1.2)
        kline.add_vol(df, row=2)
        kline.add_macd(df, row=3)

        if len(bi_list) > 0:
            bi1 = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx, "text": x.fx_a.mark.value} for x in bi_list]
            bi2 = [{'dt': bi_list[-1].fx_b.dt, "bi": bi_list[-1].fx_b.fx, "text": bi_list[-1].fx_b.mark.value[0]}]
            bi = pd.DataFrame(bi1 + bi2)
            fx = pd.DataFrame([{'dt': x.dt, "fx": x.fx} for x in self.fx_list])
            kline.add_scatter_indicator(fx['dt'], fx['fx'], name="分型", row=1, line_width=2)
            kline.add_scatter_indicator(bi['dt'], bi['bi'], name="笔", text=bi['text'], row=1, line_width=2)
        return kline.fig

    def open_in_browser(self, width: str = "1400px", height: str = '580px'):
        """直接在浏览器中打开分析结果

        :param width: 图表宽度
        :param height: 图表高度
        :return:
        """
        home_path = os.path.expanduser("~")
        file_html = os.path.join(home_path, "temp_czsc.html")
        chart = self.to_echarts(width, height)
        chart.render(file_html)
        webbrowser.open(file_html)

    @property
    def last_bi_extend(self):
        """判断最后一笔是否在延伸中，True 表示延伸中"""
        if self.bi_list[-1].direction == Direction.Up \
                and max([x.high for x in self.bars_ubi]) > self.bi_list[-1].high:
            return True

        if self.bi_list[-1].direction == Direction.Down \
                and min([x.low for x in self.bars_ubi]) < self.bi_list[-1].low:
            return True

        return False

    @property
    def finished_bis(self) -> List[BI]:
        """已完成的笔"""
        if not self.bi_list:
            return []
        if len(self.bars_ubi) < 5:
            return self.bi_list[:-1]
        return self.bi_list

    @property
    def ubi_fxs(self) -> List[FX]:
        """bars_ubi 中的分型"""
        if not self.bars_ubi:
            return []
        else:
            return check_fxs(self.bars_ubi)

    @property
    def ubi(self):
        """Unfinished Bi，未完成的笔"""
        ubi_fxs = self.ubi_fxs
        if not self.bars_ubi or not self.bi_list or not ubi_fxs:
            return None

        bars_raw = [y for x in self.bars_ubi for y in x.raw_bars]
        # 获取最高点和最低点，以及对应的时间
        high_bar = max(bars_raw, key=lambda x: x.high)
        low_bar = min(bars_raw, key=lambda x: x.low)
        direction = Direction.Up if self.bi_list[-1].direction == Direction.Down else Direction.Down

        bi = {
            "symbol": self.symbol,
            "direction": direction,
            "high": high_bar.high,
            "low": low_bar.low,
            "high_bar": high_bar,
            "low_bar": low_bar,
            "bars": self.bars_ubi,
            "raw_bars": bars_raw,
            "fxs": ubi_fxs,
            "fx_a": ubi_fxs[0],
        }
        return bi

    @property
    def fx_list(self) -> List[FX]:
        """分型列表，包括 bars_ubi 中的分型"""
        fxs = []
        for bi_ in self.bi_list:
            fxs.extend(bi_.fxs[1:])
        ubi = self.ubi_fxs
        for x in ubi:
            if not fxs or x.dt > fxs[-1].dt:
                fxs.append(x)
        return fxs



from _typeshed import Incomplete
from collections.abc import Generator
from contextlib import contextmanager
from typing import Any, Callable

@contextmanager
def capture_warnings() -> Generator[Incomplete, None, Incomplete]: ...
def execute_with_warning_capture(func: Callable, *args, return_as_string: bool = False, drop_duplicates: bool = True, **kwargs) -> tuple[list[str], Any] | tuple[str, Any]: ...



from .base import ensure_datetime_index as ensure_datetime_index, generate_component_key as generate_component_key

def show_correlation(df, key=None, **kwargs) -> None: ...
def show_sectional_ic(df, factors, target_col, key=None, **kwargs) -> None: ...
def show_ts_rolling_corr(df, col1, col2, window: int = 60, key=None, **kwargs) -> None: ...
def show_ts_self_corr(df, col, max_lag: int = 20, key=None, **kwargs) -> None: ...
def show_cointegration(df, col1, col2, key=None, **kwargs) -> None: ...
def show_corr_graph(df, threshold: float = 0.3, key=None, **kwargs) -> None: ...
def show_symbols_corr(df, symbols, key=None, **kwargs) -> None: ...



from collections import OrderedDict
from czsc.core import CZSC as CZSC
from czsc.signals.jcc import check_szx as check_szx
from czsc.signals.tas import update_ma_cache as update_ma_cache
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements

def xl_bar_position_V240328(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240329(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240330(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240331(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_basis_V240412(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_basis_V240411(c: CZSC, **kwargs) -> OrderedDict: ...
def xl_bar_trend_V240623(c: CZSC, **kwargs) -> OrderedDict: ...



"""
公共绘图函数和常量

此模块包含所有绘图模块共用的常量、类型定义和辅助函数
"""

from typing import Union, Literal, Optional
import pandas as pd
import plotly.graph_objects as go

# ==================== 模块级常量 ====================

# 颜色常量
COLOR_DRAWDOWN = "salmon"
COLOR_RETURN = "#34a853"
COLOR_ANNO_GRAY = "rgba(128,128,128,0.5)"
COLOR_ANNO_RED = "red"
COLOR_BORDER = "darkgrey"
COLOR_HEADER_BG = "grey"

# 分位数常量
QUANTILES_DRAWDOWN = [0.05, 0.1, 0.2]
QUANTILES_DRAWDOWN_ANALYSIS = [0.1, 0.3, 0.5]

# Sigma 级别
SIGMA_LEVELS = [-3, -2, -1, 1, 2, 3]

# 月份标签
MONTH_LABELS = [
    '1月', '2月', '3月', '4月', '5月', '6月',
    '7月', '8月', '9月', '10月', '11月', '12月'
]

# 模板类型
TemplateType = Literal['plotly', 'plotly_dark', 'ggplot2', 'seaborn', 'simple_white']


# ==================== 辅助函数 ====================

def figure_to_html(
    fig: go.Figure,
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """统一处理 Figure 转 HTML 的逻辑

    :param fig: Plotly Figure 对象
    :param to_html: 是否转换为 HTML
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库
    :return: Figure 对象或 HTML 字符串
    """
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def add_year_boundary_lines(
    fig: go.Figure,
    dates: pd.DatetimeIndex,
    row: Optional[int] = None,
    col: Optional[int] = None,
    line_color: str = "red",
    opacity: float = 0.3,
    line_dash: str = "dash"
) -> None:
    """在图表中添加年度分隔线

    :param fig: Plotly Figure 对象
    :param dates: 日期索引
    :param row: 子图行号（可选）
    :param col: 子图列号（可选）
    :param line_color: 线条颜色
    :param opacity: 透明度
    :param line_dash: 线条样式
    """
    years = dates.year.unique()
    for year in years:
        first_date = dates[dates.year == year].min()
        fig.add_vline(
            x=first_date,
            line_dash=line_dash,
            line_color=line_color,
            opacity=opacity,
            row=row,
            col=col
        )



import pandas as pd
import plotly.graph_objects as go
from .common import COLOR_ANNO_GRAY as COLOR_ANNO_GRAY, COLOR_ANNO_RED as COLOR_ANNO_RED, COLOR_BORDER as COLOR_BORDER, COLOR_DRAWDOWN as COLOR_DRAWDOWN, COLOR_HEADER_BG as COLOR_HEADER_BG, COLOR_RETURN as COLOR_RETURN, MONTH_LABELS as MONTH_LABELS, QUANTILES_DRAWDOWN as QUANTILES_DRAWDOWN, QUANTILES_DRAWDOWN_ANALYSIS as QUANTILES_DRAWDOWN_ANALYSIS, SIGMA_LEVELS as SIGMA_LEVELS, TemplateType as TemplateType

def plot_cumulative_returns(dret: pd.DataFrame, title: str = '累计收益', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def plot_drawdown_analysis(dret: pd.DataFrame, ret_col: str = 'total', title: str = '回撤分析', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def plot_daily_return_distribution(dret: pd.DataFrame, ret_col: str = 'total', title: str = '日收益分布', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def plot_monthly_heatmap(dret: pd.DataFrame, ret_col: str = 'total', title: str = '月度收益热力图', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def get_performance_metrics_cards(stats: dict) -> list: ...
def plot_backtest_stats(dret: pd.DataFrame, ret_col: str = 'total', title: str = '', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True) -> go.Figure | str: ...
def plot_colored_table(df: pd.DataFrame, title: str = '策略绩效统计', template: TemplateType = 'plotly_dark', float_fmt: str = '.2f', to_html: bool = False, include_plotlyjs: bool = True, **kwargs) -> go.Figure | str: ...
def plot_long_short_comparison(dailys_pivot: pd.DataFrame, stats_df: pd.DataFrame, title: str = '多空收益对比', template: TemplateType = 'plotly', to_html: bool = False, include_plotlyjs: bool = True, target_volatility: float = 0.2) -> go.Figure | str: ...



"""
绘图工具模块

统一的可视化工具，包括回测图表、权重分析图表、K线图表等
"""

# 从 backtest 导入回测相关绘图函数
from .backtest import (
    plot_cumulative_returns,
    plot_drawdown_analysis,
    plot_daily_return_distribution,
    plot_monthly_heatmap,
    plot_backtest_stats,
    plot_colored_table,
    plot_long_short_comparison,
    get_performance_metrics_cards,
)

# 从 weight 导入权重相关绘图函数
from .weight import (
    calculate_turnover_stats,
    calculate_weight_stats,
    plot_weight_histogram_kde,
    plot_weight_cdf,
    plot_turnover_overview,
    plot_turnover_cost_analysis,
)

# 从 kline 导入K线相关绘图类和函数
from .kline import (
    KlineChart,
    plot_czsc_chart,
)

# 从 common 导出常用常量
from .common import (
    COLOR_DRAWDOWN,
    COLOR_RETURN,
    COLOR_ANNO_GRAY,
    COLOR_ANNO_RED,
    COLOR_BORDER,
    COLOR_HEADER_BG,
    QUANTILES_DRAWDOWN,
    QUANTILES_DRAWDOWN_ANALYSIS,
    SIGMA_LEVELS,
    MONTH_LABELS,
    TemplateType,
    figure_to_html,
    add_year_boundary_lines,
)

__all__ = [
    # Backtest plotting functions
    'plot_cumulative_returns',
    'plot_drawdown_analysis',
    'plot_daily_return_distribution',
    'plot_monthly_heatmap',
    'plot_backtest_stats',
    'plot_colored_table',
    'plot_long_short_comparison',
    'get_performance_metrics_cards',
    # Weight plotting functions
    'calculate_turnover_stats',
    'calculate_weight_stats',
    'plot_weight_histogram_kde',
    'plot_weight_cdf',
    'plot_turnover_overview',
    'plot_turnover_cost_analysis',
    # Kline plotting
    'KlineChart',
    'plot_czsc_chart',
    # Common constants and utilities
    'COLOR_DRAWDOWN',
    'COLOR_RETURN',
    'COLOR_ANNO_GRAY',
    'COLOR_ANNO_RED',
    'COLOR_BORDER',
    'COLOR_HEADER_BG',
    'QUANTILES_DRAWDOWN',
    'QUANTILES_DRAWDOWN_ANALYSIS',
    'SIGMA_LEVELS',
    'MONTH_LABELS',
    'TemplateType',
    'figure_to_html',
    'add_year_boundary_lines',
]



from _typeshed import Incomplete
from czsc.fsa.base import FeishuApiBase as FeishuApiBase, request as request

class BiTable(FeishuApiBase):
    app_token: Incomplete
    def __init__(self, app_id=None, app_secret=None, app_token=None, **kwargs) -> None: ...
    def one_record(self, table_id, record_id): ...
    def list_records(self, table_id, **kwargs): ...
    def create_table(self, name=None, default_view_name=None, fields=None): ...
    def batch_create_table(self, names=None): ...
    def delete_table(self, table_id): ...
    def batch_delete_table(self, table_ids=None): ...
    def patch_table(self, table_id, name): ...
    def list_tables(self, page_token=None, page_size: int = 20): ...
    def table_record_get(self, table_id, record_id, text_field_as_array=None, user_id_type=None, display_formula_ref=None, with_shared_url=None, automatic_fields=None): ...
    def table_record_search(self, table_id, user_id_type=None, page_token=None, page_size: int = 20, view_id=None, field_names=None, sort=None, filter=None, automatic_fields=None): ...
    def table_record_create(self, table_id, fields, user_id_type=None, client_token=None): ...
    def table_record_update(self, table_id, record_id, fields, user_id_type=None): ...
    def table_record_delete(self, table_id, record_id): ...
    def table_record_batch_create(self, table_id, records, user_id_type=None, client_token=None): ...
    def table_record_batch_update(self, table_id, records, user_id_type=None): ...
    def table_record_batch_delete(self, table_id, record_ids): ...
    def table_view_patch(self, table_id, view_id, infos): ...
    def table_view_get(self, table_id, view_id): ...
    def table_view_list(self, table_id, page_size: int = 20, user_id_type=None, page_token=None): ...
    def table_view_create(self, table_id, view_name, view_type): ...
    def table_view_delete(self, table_id, view_id): ...
    def table_field_list(self, table_id, page_size: int = 20, view_id=None, text_field_as_array=None, page_token=None): ...
    def table_field_create(self, table_id, field_name, type, property=None, description=None, ui_type=None, client_token=None): ...
    def table_field_update(self, table_id, field_id, field_name, type, property=None, description=None, ui_type=None): ...
    def table_field_delete(self, table_id, field_id): ...
    def table_form_patch_2(self, table_id, form_id, name=None, description=None, shared=None, shared_limit=None, submit_limit_once=None): ...
    def table_form_get(self, table_id, form_id): ...
    def table_form_patch(self, table_id, form_id, field_id, pre_field_id=None, title=None, description=None, required=None, visible=None): ...
    def table_form_list(self, table_id, form_id): ...
    def table_copy(self, app_token=None, name=None, folder_token=None, without_content=None, time_zone=None): ...
    def table_create(self, name=None, folder_token=None, time_zone=None): ...
    def table_get(self, app_token=None): ...
    def table_update(self, app_token=None, name=None, is_advanced=None): ...
    @property
    def tables(self): ...
    def read_table(self, table_id, **kwargs): ...
    def empty_table(self, table_id, **kwargs) -> None: ...



from _typeshed import Incomplete
from czsc import envs as envs
from czsc.py.enum import Direction as Direction, Mark as Mark
from czsc.py.objects import BI as BI, FX as FX, NewBar as NewBar, RawBar as RawBar

def remove_include(k1: NewBar, k2: NewBar, k3: RawBar): ...
def check_fx(k1: NewBar, k2: NewBar, k3: NewBar): ...
def check_fxs(bars: list[NewBar]) -> list[FX]: ...
def check_bi(bars: list[NewBar], **kwargs): ...

class CZSC:
    verbose: Incomplete
    max_bi_num: Incomplete
    bars_raw: list[RawBar]
    bars_ubi: list[NewBar]
    bi_list: list[BI]
    symbol: Incomplete
    freq: Incomplete
    get_signals: Incomplete
    signals: Incomplete
    cache: Incomplete
    def __init__(self, bars: list[RawBar], get_signals=None, max_bi_num=...) -> None: ...
    def update(self, bar: RawBar): ...
    def to_echarts(self, width: str = '1400px', height: str = '580px', bs=[]): ...
    def to_plotly(self): ...
    def open_in_browser(self, width: str = '1400px', height: str = '580px'): ...
    @property
    def last_bi_extend(self): ...
    @property
    def finished_bis(self) -> list[BI]: ...
    @property
    def ubi_fxs(self) -> list[FX]: ...
    @property
    def ubi(self): ...
    @property
    def fx_list(self) -> list[FX]: ...



import pandas as pd
from typing import Any

def to_standard_kline_format(df: pd.DataFrame, dt_col: str = 'dt', open_col: str = 'open', high_col: str = 'high', low_col: str = 'low', close_col: str = 'close', volume_col: str = 'vol') -> pd.DataFrame: ...
def pivot_weight_data(dfw: pd.DataFrame, fill_value: float = 0.0) -> pd.DataFrame: ...
def resample_to_period(df: pd.DataFrame, period: str, agg_dict: dict[str, str] | None = None) -> pd.DataFrame: ...
def normalize_symbol(symbol: str) -> str: ...
def convert_dict_to_dataframe(data: list[dict[str, Any]], sort_by: str | None = None) -> pd.DataFrame: ...
def ensure_datetime_column(df: pd.DataFrame, column: str, inplace: bool = False) -> pd.DataFrame: ...
def flatten_multiindex_columns(df: pd.DataFrame, sep: str = '_') -> pd.DataFrame: ...



from _typeshed import Incomplete
from pathlib import Path
from typing import Any, AnyStr

home_path: Incomplete

def get_dir_size(path): ...
def empty_cache_path() -> None: ...

class DiskCache:
    path: Incomplete
    def __init__(self, path=None) -> None: ...
    def is_found(self, k: str, suffix: str = 'pkl', ttl: int = -1) -> bool: ...
    def get(self, k: str, suffix: str = 'pkl') -> Any: ...
    def set(self, k: str, v: Any, suffix: str = 'pkl'): ...
    def remove(self, k: str, suffix: str = 'pkl'): ...

def disk_cache(path: AnyStr | Path = ..., suffix: str = 'pkl', ttl: int = -1): ...
def clear_expired_cache(path, max_age: int = ...): ...
def clear_cache(path: AnyStr | Path = ..., subs=None, recreate: bool = False): ...



import abc
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from czsc.core import BarGenerator as BarGenerator, Position as Position, RawBar as RawBar, Signal as Signal
from czsc.py.bar_generator import check_freq_and_market as check_freq_and_market
from czsc.traders.base import CzscTrader as CzscTrader
from czsc.traders.sig_parse import get_signals_config as get_signals_config, get_signals_freqs as get_signals_freqs
from czsc.utils.io import dill_dump as dill_dump, read_json as read_json, save_json as save_json

class CzscStrategyBase(ABC, metaclass=abc.ABCMeta):
    kwargs: Incomplete
    signals_module_name: Incomplete
    def __init__(self, **kwargs) -> None: ...
    @property
    def symbol(self): ...
    @property
    def unique_signals(self): ...
    @property
    def signals_config(self): ...
    @property
    def freqs(self): ...
    @property
    def sorted_freqs(self): ...
    @property
    def base_freq(self): ...
    @abstractmethod
    def positions(self) -> list[Position]: ...
    def init_bar_generator(self, bars: list[RawBar], **kwargs): ...
    def init_trader(self, bars: list[RawBar], **kwargs) -> CzscTrader: ...
    def backtest(self, bars: list[RawBar], **kwargs) -> CzscTrader: ...
    def dummy(self, sigs: list[dict], **kwargs) -> CzscTrader: ...
    def replay(self, bars: list[RawBar], res_path, **kwargs): ...
    def check(self, bars: list[RawBar], res_path, **kwargs): ...
    def save_positions(self, path) -> None: ...
    def load_positions(self, files: list, check: bool = True) -> list[Position]: ...

class CzscJsonStrategy(CzscStrategyBase):
    @property
    def positions(self): ...



from collections import OrderedDict
from czsc import CZSC as CZSC
from czsc.core import BI as BI, Direction as Direction, Mark as Mark
from czsc.signals.tas import update_boll_cache_V230228 as update_boll_cache_V230228, update_ma_cache as update_ma_cache, update_macd_cache as update_macd_cache
from czsc.utils.sig import create_single_signal as create_single_signal, get_sub_elements as get_sub_elements, is_symmetry_zs as is_symmetry_zs

def byi_symmetry_zs_V221107(c: CZSC, **kwargs): ...
def byi_bi_end_V230106(c: CZSC, **kwargs) -> OrderedDict: ...
def byi_bi_end_V230107(c: CZSC, **kwargs) -> OrderedDict: ...
def byi_second_bs_V230324(c: CZSC, **kwargs) -> OrderedDict: ...
def byi_fx_num_V230628(c: CZSC, **kwargs) -> OrderedDict: ...
